# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------

# pylint: disable=line-too-long
from azure.cli.core.commands import CliCommandType
from azure.cli.core.util import empty_on_404

from ._client_factory import cf_web_client, cf_plans, cf_webapps
from ._validators import (validate_onedeploy_params, validate_staticsite_link_function, validate_staticsite_sku,
                          validate_vnet_integration, validate_asp_create, validate_functionapp_asp_create,
                          validate_webapp_up, validate_app_exists, validate_add_vnet, validate_app_is_functionapp,
                          validate_app_is_webapp, validate_functionapp_on_containerapp_vnet,
                          validate_functionapp_on_containerapp_vnet_add, validate_centauri_delete_function,
                          validate_functionapp_on_containerapp_site_config_set,
                          validate_functionapp_on_containerapp_site_config_show,
                          validate_functionapp_on_containerapp_container_settings_delete,
                          validate_functionapp_on_containerapp_update, validate_functionapp_on_flex_plan,
                          validate_functionapp, validate_is_flex_functionapp)


def output_slots_in_table(slots):
    return [{'name': s['name'], 'status': s['state'], 'plan': s['appServicePlan']} for s in slots]


def transform_list_location_output(result):
    return [{'name': x.name} for x in result]


def transform_web_output(web):
    props = ['name', 'state', 'location', 'resourceGroup', 'defaultHostName', 'appServicePlanId', 'ftpPublishingUrl']
    result = {k: web[k] for k in web if k in props}
    # to get width under control, also the plan usually is in the same RG
    result['appServicePlan'] = result.pop('appServicePlanId').split('/')[-1]
    return result


def transform_web_list_output(webs):
    return [transform_web_output(w) for w in webs]


def ex_handler_factory(creating_plan=False):
    def _ex_handler(ex):
        ex = _polish_bad_errors(ex, creating_plan)
        raise ex
    return _ex_handler


def update_function_ex_handler_factory():
    from azure.cli.core.azclierror import ClientRequestError

    def _ex_handler(ex):
        http_error_response = False
        if hasattr(ex, 'response'):
            http_error_response = True
        ex = _polish_bad_errors(ex, False)
        # only include if an update was attempted and failed on the backend
        if http_error_response:
            try:
                detail = ('If using \'--plan\', a consumption plan may be unable to migrate '
                          'to a given premium plan. Please confirm that the premium plan '
                          'exists in the same resource group and region. Note: Not all '
                          'functionapp plans support premium instances. If you have verified '
                          'your resource group and region and are still unable to migrate, '
                          'please redeploy on a premium functionapp plan.')
                ex = ClientRequestError(ex.args[0] + '\n\n' + detail)
            except Exception:  # pylint: disable=broad-except
                pass
        raise ex
    return _ex_handler


def _polish_bad_errors(ex, creating_plan):
    import json
    from knack.util import CLIError
    try:
        if hasattr(ex, "response"):
            if 'text/plain' in ex.response.headers['Content-Type']:  # HTML Response
                detail = ex.response.text
            else:
                detail = json.loads(ex.response.text())['Message']
                if creating_plan:
                    if 'Requested features are not supported in region' in detail:
                        detail = ("Plan with requested features is not supported in current region. \n"
                                  "If creating an App Service Plan with --zone-redundant/-z, "
                                  "please see supported regions here: "
                                  "https://docs.microsoft.com/en-us/azure/app-service/how-to-zone-redundancy#requirements")
        else:
            detail = json.loads(ex.error_msg.response.text())['Message']
        ex = CLIError(detail)
    except Exception:  # pylint: disable=broad-except
        pass
    return ex


# pylint: disable=too-many-statements
def load_command_table(self, _):
    webclient_sdk = CliCommandType(
        operations_tmpl='azure.mgmt.web.operations#WebSiteManagementClientOperationsMixin.{}',
        client_factory=cf_web_client
    )
    appservice_plan_sdk = CliCommandType(
        operations_tmpl='azure.mgmt.web.operations#AppServicePlansOperations.{}',
        client_factory=cf_plans
    )
    webapp_sdk = CliCommandType(
        operations_tmpl='azure.mgmt.web.operations#WebAppsOperations.{}',
        client_factory=cf_webapps
    )

    appservice_custom = CliCommandType(operations_tmpl='azure.cli.command_modules.appservice.custom#{}')

    webapp_access_restrictions = CliCommandType(operations_tmpl='azure.cli.command_modules.appservice.access_restrictions#{}')

    appservice_environment = CliCommandType(operations_tmpl='azure.cli.command_modules.appservice.appservice_environment#{}')

    staticsite_sdk = CliCommandType(operations_tmpl='azure.cli.command_modules.appservice.static_sites#{}')

    appservice_domains = CliCommandType(operations_tmpl='azure.cli.command_modules.appservice.appservice_domains#{}')

    logicapp_custom = CliCommandType(operations_tmpl='azure.cli.command_modules.appservice.logicapp.custom#{}')

    with self.command_group('webapp', webapp_sdk) as g:
        g.custom_command('create', 'create_webapp', exception_handler=ex_handler_factory(), validator=validate_vnet_integration)
        g.custom_command('up', 'webapp_up', exception_handler=ex_handler_factory(), validator=validate_webapp_up)
        g.custom_command('ssh', 'ssh_webapp', exception_handler=ex_handler_factory(), is_preview=True)
        g.custom_command('list', 'list_webapp', table_transformer=transform_web_list_output)
        g.custom_show_command('show', 'show_app', table_transformer=transform_web_output)
        g.custom_command('delete', 'delete_webapp')
        g.custom_command('stop', 'stop_webapp')
        g.custom_command('start', 'start_webapp')
        g.custom_command('restart', 'restart_webapp')
        g.custom_command('browse', 'view_in_browser')
        g.custom_command('list-instances', 'list_instances')
        g.custom_command('list-runtimes', 'list_runtimes')
        g.custom_command('identity assign', 'assign_identity')
        g.custom_show_command('identity show', 'show_identity')
        g.custom_command('identity remove', 'remove_identity')
        g.custom_command('create-remote-connection', 'create_tunnel', exception_handler=ex_handler_factory())
        g.custom_command('deploy', 'perform_onedeploy_webapp', validator=validate_onedeploy_params)
        g.generic_update_command('update', getter_name='get_webapp', setter_name='set_webapp',
                                 custom_func_name='update_webapp', command_type=appservice_custom)

    with self.command_group('webapp traffic-routing') as g:
        g.custom_command('set', 'set_traffic_routing')
        g.custom_show_command('show', 'show_traffic_routing')
        g.custom_command('clear', 'clear_traffic_routing')

    with self.command_group('webapp cors') as g:
        g.custom_command('add', 'add_cors')
        g.custom_command('remove', 'remove_cors')
        g.custom_show_command('show', 'show_cors')

    with self.command_group('webapp config') as g:
        g.custom_command('set', 'update_site_configs')
        g.custom_show_command('show', 'get_site_configs')

    with self.command_group('webapp config appsettings') as g:
        g.custom_command('list', 'get_app_settings', exception_handler=empty_on_404)
        g.custom_command('set', 'update_app_settings')
        g.custom_command('delete', 'delete_app_settings')

    with self.command_group('webapp config connection-string') as g:
        g.custom_command('list', 'get_connection_strings', exception_handler=empty_on_404)
        g.custom_command('set', 'update_connection_strings')
        g.custom_command('delete', 'delete_connection_strings')

    with self.command_group('webapp config storage-account') as g:
        g.custom_command('list', 'get_azure_storage_accounts', exception_handler=empty_on_404)
        g.custom_command('add', 'add_azure_storage_account')
        g.custom_command('update', 'update_azure_storage_account')
        g.custom_command('delete', 'delete_azure_storage_accounts')

    with self.command_group('webapp config hostname') as g:
        g.custom_command('add', 'add_hostname', exception_handler=ex_handler_factory())
        g.custom_command('list', 'list_hostnames')
        g.custom_command('delete', 'delete_hostname')
        g.custom_command('get-external-ip', 'get_external_ip')

    with self.command_group('webapp config container') as g:
        g.custom_command('set', 'update_container_settings')
        g.custom_command('delete', 'delete_container_settings')
        g.custom_show_command('show', 'show_container_settings')

    with self.command_group('webapp config ssl') as g:
        g.custom_command('upload', 'upload_ssl_cert')
        g.custom_command('list', 'list_ssl_certs', exception_handler=ex_handler_factory())
        g.custom_show_command('show', 'show_ssl_cert', exception_handler=ex_handler_factory())
        g.custom_command('bind', 'bind_ssl_cert', exception_handler=ex_handler_factory())
        g.custom_command('unbind', 'unbind_ssl_cert')
        g.custom_command('delete', 'delete_ssl_cert', exception_handler=ex_handler_factory())
        g.custom_command('import', 'import_ssl_cert', exception_handler=ex_handler_factory())
        g.custom_command('create', 'create_managed_ssl_cert', exception_handler=ex_handler_factory(), is_preview=True)

    with self.command_group('webapp config backup') as g:
        g.custom_command('list', 'list_backups')
        g.custom_show_command('show', 'show_backup_configuration')
        g.custom_command('create', 'create_backup', exception_handler=ex_handler_factory())
        g.custom_command('update', 'update_backup_schedule', exception_handler=ex_handler_factory())
        g.custom_command('restore', 'restore_backup', exception_handler=ex_handler_factory())
        g.custom_command('delete', 'delete_backup', exception_handler=ex_handler_factory(), confirmation=True)

    with self.command_group('webapp config snapshot') as g:
        g.custom_command('list', 'list_snapshots')
        g.custom_command('restore', 'restore_snapshot')

    with self.command_group('webapp webjob continuous') as g:
        g.custom_command('list', 'list_continuous_webjobs', exception_handler=ex_handler_factory())
        g.custom_command('remove', 'remove_continuous_webjob', exception_handler=ex_handler_factory())
        g.custom_command('start', 'start_continuous_webjob', exception_handler=ex_handler_factory())
        g.custom_command('stop', 'stop_continuous_webjob', exception_handler=ex_handler_factory())

    with self.command_group('webapp webjob triggered') as g:
        g.custom_command('list', 'list_triggered_webjobs', exception_handler=ex_handler_factory())
        g.custom_command('remove', 'remove_triggered_webjob', exception_handler=ex_handler_factory())
        g.custom_command('run', 'run_triggered_webjob', exception_handler=ex_handler_factory())
        g.custom_command('log', 'get_history_triggered_webjob', exception_handler=ex_handler_factory())

    with self.command_group('webapp deployment source') as g:
        g.custom_command('config-local-git', 'enable_local_git')
        g.custom_command('config-zip', 'enable_zip_deploy_webapp', exception_handler=ex_handler_factory(), deprecate_info=g.deprecate(redirect='az webapp deploy', hide=True))
        g.custom_command('config', 'config_source_control', exception_handler=ex_handler_factory())
        g.custom_command('sync', 'sync_site_repo', exception_handler=ex_handler_factory())
        g.custom_show_command('show', 'show_source_control')
        g.custom_command('delete', 'delete_source_control')
        g.custom_command('update-token', 'update_git_token', exception_handler=ex_handler_factory())

    with self.command_group('webapp log') as g:
        g.custom_command('tail', 'get_streaming_log', validator=validate_app_exists)
        g.custom_command('download', 'download_historical_logs')
        g.custom_command('config', 'config_diagnostics')
        g.custom_show_command('show', 'show_diagnostic_settings')

    with self.command_group('webapp log deployment') as g:
        g.custom_show_command('show', 'show_deployment_log')
        g.custom_command('list', 'list_deployment_logs')

    with self.command_group('functionapp log deployment') as g:
        g.custom_show_command('show', 'show_deployment_log')
        g.custom_command('list', 'list_deployment_logs')

    with self.command_group('webapp deployment slot') as g:
        g.custom_command('list', 'list_slots', table_transformer=output_slots_in_table)
        g.custom_command('delete', 'delete_slot')
        g.custom_command('auto-swap', 'config_slot_auto_swap')
        g.custom_command('swap', 'swap_slot', exception_handler=ex_handler_factory())
        g.custom_command('create', 'create_webapp_slot', exception_handler=ex_handler_factory())

    with self.command_group('webapp deployment') as g:
        g.custom_command('list-publishing-profiles', 'list_publish_profiles')
        g.custom_command('list-publishing-credentials', 'list_publishing_credentials')

    with self.command_group('webapp deployment user', webclient_sdk) as g:
        g.custom_show_command('show', 'get_publishing_user')
        g.custom_command('set', 'set_deployment_user', exception_handler=ex_handler_factory())

    with self.command_group('webapp deployment container') as g:
        g.custom_command('config', 'enable_cd')
        g.custom_command('show-cd-url', 'show_container_cd_url')

    with self.command_group('webapp deployment github-actions', validator=validate_app_is_webapp) as g:
        g.custom_command('add', 'add_github_actions')
        g.custom_command('remove', 'remove_github_actions')

    with self.command_group('functionapp deployment github-actions', validator=validate_app_is_functionapp) as g:
        g.custom_command('add', 'add_functionapp_github_actions')
        g.custom_command('remove', 'remove_functionapp_github_actions')

    with self.command_group('webapp auth') as g:
        g.custom_show_command('show', 'get_auth_settings')
        g.custom_command('update', 'update_auth_settings')

    with self.command_group('webapp deleted', is_preview=True) as g:
        g.custom_command('list', 'list_deleted_webapp')
        g.custom_command('restore', 'restore_deleted_webapp')

    with self.command_group('webapp hybrid-connection') as g:
        g.custom_command('list', 'list_hc')
        g.custom_command('add', 'add_hc')
        g.custom_command('remove', 'remove_hc')

    with self.command_group('functionapp hybrid-connection') as g:
        g.custom_command('list', 'list_hc')
        g.custom_command('add', 'add_hc')
        g.custom_command('remove', 'remove_hc')

    with self.command_group('appservice hybrid-connection') as g:
        g.custom_command('set-key', 'set_hc_key')

    with self.command_group('webapp vnet-integration') as g:
        g.custom_command('add', 'add_webapp_vnet_integration', validator=validate_add_vnet, exception_handler=ex_handler_factory())
        g.custom_command('list', 'list_vnet_integration')
        g.custom_command('remove', 'remove_vnet_integration')

    with self.command_group('functionapp vnet-integration') as g:
        g.custom_command('add', 'add_functionapp_vnet_integration', validator=validate_functionapp_on_containerapp_vnet_add, exception_handler=ex_handler_factory())
        g.custom_command('list', 'list_functionapp_vnet_integration', validator=validate_functionapp_on_containerapp_vnet, exception_handler=ex_handler_factory())
        g.custom_command('remove', 'remove_functionapp_vnet_integration', validator=validate_functionapp_on_containerapp_vnet, exception_handler=ex_handler_factory())

    with self.command_group('appservice plan', appservice_plan_sdk) as g:
        g.custom_command('create', 'create_app_service_plan', supports_no_wait=True,
                         exception_handler=ex_handler_factory(creating_plan=True), validator=validate_asp_create)
        g.command('delete', 'delete', confirmation=True)
        g.custom_command('list', 'list_app_service_plans')
        g.custom_show_command('show', 'show_plan')
        g.generic_update_command('update', setter_name='begin_create_or_update', custom_func_name='update_app_service_plan',
                                 setter_arg_name='app_service_plan', supports_no_wait=True,
                                 exception_handler=ex_handler_factory())

    with self.command_group('appservice') as g:
        g.custom_command('list-locations', 'list_locations', transform=transform_list_location_output)

    with self.command_group('appservice vnet-integration') as g:
        g.custom_command('list', 'appservice_list_vnet')

    with self.command_group('functionapp') as g:
        g.custom_command('create', 'create_functionapp', exception_handler=ex_handler_factory(),
                         validator=validate_functionapp)
        g.custom_command('list-runtimes', 'list_function_app_runtimes')
        g.custom_command('list-flexconsumption-runtimes', 'list_flexconsumption_runtimes')
        g.custom_command('list', 'list_function_app', table_transformer=transform_web_list_output)
        g.custom_show_command('show', 'show_functionapp', table_transformer=transform_web_output)
        g.custom_command('delete', 'delete_function_app')
        g.custom_command('stop', 'stop_webapp')
        g.custom_command('start', 'start_webapp')
        g.custom_command('restart', 'restart_webapp')
        g.custom_command('list-consumption-locations', 'list_consumption_locations')
        g.custom_command('list-flexconsumption-locations', 'list_flexconsumption_locations')
        g.custom_command('identity assign', 'assign_identity')
        g.custom_show_command('identity show', 'show_identity')
        g.custom_command('identity remove', 'remove_identity')
        g.custom_command('deploy', 'perform_onedeploy_functionapp', validator=validate_onedeploy_params, is_preview=True)
        g.generic_update_command('update', getter_name="get_functionapp", setter_name='set_functionapp', exception_handler=update_function_ex_handler_factory(),
                                 custom_func_name='update_functionapp', getter_type=appservice_custom, setter_type=appservice_custom, command_type=webapp_sdk,
                                 validator=validate_functionapp_on_containerapp_update)

    with self.command_group('functionapp deployment config') as g:
        g.custom_command('set', 'update_deployment_configs', exception_handler=ex_handler_factory(), validator=validate_is_flex_functionapp)
        g.custom_show_command('show', 'get_deployment_configs', exception_handler=ex_handler_factory(), validator=validate_is_flex_functionapp)

    with self.command_group('functionapp config') as g:
        g.custom_command('set', 'update_site_configs_functionapp', validator=validate_functionapp_on_containerapp_site_config_set, exception_handler=ex_handler_factory())
        g.custom_show_command('show', 'get_site_configs', validator=validate_functionapp_on_containerapp_site_config_show, exception_handler=ex_handler_factory())

    with self.command_group('functionapp config appsettings') as g:
        g.custom_command('list', 'get_app_settings', exception_handler=empty_on_404)
        g.custom_command('set', 'update_app_settings_functionapp', exception_handler=ex_handler_factory())
        g.custom_command('delete', 'delete_app_settings', exception_handler=ex_handler_factory())

    with self.command_group('functionapp runtime config') as g:
        g.custom_show_command('show', 'get_runtime_config', exception_handler=ex_handler_factory(), validator=validate_is_flex_functionapp)
        g.custom_command('set', 'update_runtime_config', exception_handler=ex_handler_factory(), validator=validate_is_flex_functionapp)

    with self.command_group('functionapp scale config') as g:
        g.custom_show_command('show', 'get_scale_config', exception_handler=ex_handler_factory(), validator=validate_is_flex_functionapp)
        g.custom_command('set', 'update_scale_config', exception_handler=ex_handler_factory(), validator=validate_is_flex_functionapp)

    with self.command_group('functionapp scale config always-ready') as g:
        g.custom_command('delete', 'delete_always_ready_settings', exception_handler=ex_handler_factory(), validator=validate_is_flex_functionapp)
        g.custom_command('set', 'update_always_ready_settings', exception_handler=ex_handler_factory(), validator=validate_is_flex_functionapp)

    with self.command_group('functionapp config hostname') as g:
        g.custom_command('add', 'add_hostname', exception_handler=ex_handler_factory())
        g.custom_command('list', 'list_hostnames')
        g.custom_command('delete', 'delete_hostname')
        g.custom_command('get-external-ip', 'get_external_ip')

    with self.command_group('functionapp config ssl') as g:
        g.custom_command('upload', 'upload_ssl_cert', exception_handler=ex_handler_factory())
        g.custom_command('list', 'list_ssl_certs')
        g.custom_show_command('show', 'show_ssl_cert')
        g.custom_command('bind', 'bind_ssl_cert', exception_handler=ex_handler_factory())
        g.custom_command('unbind', 'unbind_ssl_cert')
        g.custom_command('delete', 'delete_ssl_cert')
        g.custom_command('import', 'import_ssl_cert', exception_handler=ex_handler_factory())
        g.custom_command('create', 'create_managed_ssl_cert', exception_handler=ex_handler_factory(), is_preview=True)

    with self.command_group('functionapp deployment source') as g:
        g.custom_command('config-local-git', 'enable_local_git', validator=validate_functionapp_on_flex_plan)
        g.custom_command('config-zip', 'enable_zip_deploy_functionapp')
        g.custom_command('config', 'config_source_control', exception_handler=ex_handler_factory(), validator=validate_functionapp_on_flex_plan)
        g.custom_command('sync', 'sync_site_repo', exception_handler=ex_handler_factory(), validator=validate_functionapp_on_flex_plan)
        g.custom_show_command('show', 'show_source_control', exception_handler=ex_handler_factory(), validator=validate_functionapp_on_flex_plan)
        g.custom_command('delete', 'delete_source_control', exception_handler=ex_handler_factory(), validator=validate_functionapp_on_flex_plan)
        g.custom_command('update-token', 'update_git_token', exception_handler=ex_handler_factory())

    with self.command_group('functionapp deployment user', webclient_sdk) as g:
        g.custom_command('set', 'set_deployment_user', exception_handler=ex_handler_factory())
        g.custom_show_command('show', 'get_publishing_user')

    with self.command_group('functionapp deployment') as g:
        g.custom_command('list-publishing-profiles', 'list_publish_profiles', validator=validate_functionapp_on_flex_plan)
        g.custom_command('list-publishing-credentials', 'list_publishing_credentials', validator=validate_functionapp_on_flex_plan)

    with self.command_group('functionapp cors') as g:
        g.custom_command('add', 'add_cors')
        g.custom_command('remove', 'remove_cors')
        g.custom_show_command('show', 'show_cors')
        g.custom_command('credentials', 'enable_credentials')

    with self.command_group('functionapp plan', appservice_plan_sdk) as g:
        g.custom_command('create', 'create_functionapp_app_service_plan',
                         exception_handler=ex_handler_factory(creating_plan=True),
                         validator=validate_functionapp_asp_create)
        g.generic_update_command('update', setter_name='begin_create_or_update',
                                 custom_func_name='update_functionapp_app_service_plan',
                                 setter_arg_name='app_service_plan', exception_handler=ex_handler_factory())
        g.command('delete', 'delete', confirmation=True)
        g.custom_command('list', 'list_app_service_plans')
        g.show_command('show', 'get')

    with self.command_group('functionapp deployment container') as g:
        g.custom_command('config', 'enable_cd')
        g.custom_command('show-cd-url', 'show_container_cd_url')

    with self.command_group('functionapp config container') as g:
        g.custom_command('set', 'update_container_settings_functionapp', validator=validate_functionapp_on_flex_plan)
        g.custom_command('delete', 'delete_container_settings', validator=validate_functionapp_on_containerapp_container_settings_delete)
        g.custom_show_command('show', 'show_container_settings_functionapp', validator=validate_functionapp_on_flex_plan)

    with self.command_group('functionapp deployment slot') as g:
        g.custom_command('list', 'list_slots', table_transformer=output_slots_in_table, validator=validate_functionapp_on_flex_plan)
        g.custom_command('delete', 'delete_slot', validator=validate_functionapp_on_flex_plan)
        g.custom_command('auto-swap', 'config_slot_auto_swap', validator=validate_functionapp_on_flex_plan)
        g.custom_command('swap', 'swap_slot', exception_handler=ex_handler_factory(), validator=validate_functionapp_on_flex_plan)
        g.custom_command('create', 'create_functionapp_slot', exception_handler=ex_handler_factory(), validator=validate_functionapp_on_flex_plan)

    with self.command_group('functionapp keys') as g:
        g.custom_command('set', 'update_host_key')
        g.custom_command('list', 'list_host_keys')
        g.custom_command('delete', 'delete_host_key')

    with self.command_group('functionapp function') as g:
        g.custom_command('show', 'show_function')  # pylint: disable=show-command
        g.custom_command('delete', 'delete_function', validator=validate_centauri_delete_function, exception_handler=ex_handler_factory())
        g.custom_command('list', 'list_functions')

    with self.command_group('functionapp function keys') as g:
        g.custom_command('set', 'update_function_key')
        g.custom_command('list', 'list_function_keys')
        g.custom_command('delete', 'delete_function_key')

    with self.command_group('webapp config access-restriction', custom_command_type=webapp_access_restrictions) as g:
        g.custom_show_command('show', 'show_webapp_access_restrictions')
        g.custom_command('add', 'add_webapp_access_restriction')
        g.custom_command('remove', 'remove_webapp_access_restriction')
        g.custom_command('set', 'set_webapp_access_restriction')

    with self.command_group('functionapp config access-restriction', custom_command_type=webapp_access_restrictions) as g:
        g.custom_show_command('show', 'show_webapp_access_restrictions')
        g.custom_command('add', 'add_webapp_access_restriction')
        g.custom_command('remove', 'remove_webapp_access_restriction')
        g.custom_command('set', 'set_webapp_access_restriction')

    with self.command_group('appservice ase', custom_command_type=appservice_environment) as g:
        g.custom_command('list', 'list_appserviceenvironments')
        g.custom_command('list-addresses', 'list_appserviceenvironment_addresses')
        g.custom_command('list-plans', 'list_appserviceenvironment_plans')
        g.custom_show_command('show', 'show_appserviceenvironment')
        g.custom_command('create', 'create_appserviceenvironment_arm', supports_no_wait=True)
        g.custom_command('update', 'update_appserviceenvironment', supports_no_wait=True)
        g.custom_command('delete', 'delete_appserviceenvironment', supports_no_wait=True, confirmation=True)
        g.custom_command('create-inbound-services', 'create_ase_inbound_services', is_preview=True)
        g.custom_command('upgrade', 'upgrade_appserviceenvironment', supports_no_wait=True, confirmation=True, is_preview=True)
        g.custom_command('send-test-notification', 'send_test_notification_appserviceenvironment', is_preview=True)

    with self.command_group('appservice domain', custom_command_type=appservice_domains, is_preview=True) as g:
        g.custom_command('create', 'create_domain')
        g.custom_command('show-terms', 'show_domain_purchase_terms')

    with self.command_group('staticwebapp', custom_command_type=staticsite_sdk) as g:
        g.custom_command('list', 'list_staticsites')
        g.custom_show_command('show', 'show_staticsite')
        g.custom_command('create', 'create_staticsites', supports_no_wait=True, exception_handler=ex_handler_factory())
        g.custom_command('delete', 'delete_staticsite', supports_no_wait=True, confirmation=True)
        g.custom_command('disconnect', 'disconnect_staticsite', supports_no_wait=True)
        g.custom_command('reconnect', 'reconnect_staticsite', supports_no_wait=True)
        g.custom_command('update', 'update_staticsite', supports_no_wait=True)

    with self.command_group('staticwebapp environment', custom_command_type=staticsite_sdk) as g:
        g.custom_command('list', 'list_staticsite_environments')
        g.custom_show_command('show', 'show_staticsite_environment')
        g.custom_command('functions', 'list_staticsite_functions')
        g.custom_command('delete', 'delete_staticsite_environment', confirmation=True)

    with self.command_group('staticwebapp hostname', custom_command_type=staticsite_sdk) as g:
        g.custom_command('list', 'list_staticsite_domains')
        g.custom_command('set', 'set_staticsite_domain', supports_no_wait=True, exception_handler=ex_handler_factory())
        g.custom_command('delete', 'delete_staticsite_domain', supports_no_wait=True, confirmation=True)
        g.custom_show_command('show', 'get_staticsite_domain')

    with self.command_group('staticwebapp identity', custom_command_type=staticsite_sdk) as g:
        g.custom_command('assign', 'assign_identity', exception_handler=ex_handler_factory())
        g.custom_command('remove', 'remove_identity', confirmation=True)
        g.custom_show_command('show', 'show_identity')

    with self.command_group('staticwebapp appsettings', custom_command_type=staticsite_sdk) as g:
        g.custom_command('list', 'list_staticsite_app_settings')
        g.custom_command('set', 'set_staticsite_app_settings')
        g.custom_command('delete', 'delete_staticsite_app_settings')

    with self.command_group('staticwebapp users', custom_command_type=staticsite_sdk) as g:
        g.custom_command('list', 'list_staticsite_users')
        g.custom_command('invite', 'invite_staticsite_users')
        g.custom_command('update', 'update_staticsite_users')

    with self.command_group('staticwebapp secrets', custom_command_type=staticsite_sdk) as g:
        g.custom_command('list', 'list_staticsite_secrets')
        g.custom_command('reset-api-key', 'reset_staticsite_api_key', supports_no_wait=True)

    with self.command_group('staticwebapp functions', custom_command_type=staticsite_sdk) as g:
        g.custom_command('link', 'link_user_function', validator=validate_staticsite_link_function)
        g.custom_command('unlink', 'unlink_user_function', validator=validate_staticsite_sku)
        g.custom_show_command('show', 'get_user_function', validator=validate_staticsite_sku)

    with self.command_group('staticwebapp backends', custom_command_type=staticsite_sdk) as g:
        g.custom_command('validate', 'validate_backend', validator=validate_staticsite_sku, exception_handler=ex_handler_factory())
        g.custom_command('link', 'link_backend', validator=validate_staticsite_sku, exception_handler=ex_handler_factory())
        g.custom_command('unlink', 'unlink_backend', validator=validate_staticsite_sku, exception_handler=ex_handler_factory())
        g.custom_show_command('show', 'get_backend', validator=validate_staticsite_sku)

    with self.command_group('staticwebapp enterprise-edge', custom_command_type=staticsite_sdk) as g:
        g.custom_command('enable', 'enable_staticwebapp_enterprise_edge')
        g.custom_command('disable', 'disable_staticwebapp_enterprise_edge')
        g.custom_show_command('show', 'show_staticwebapp_enterprise_edge_status')

    with self.command_group('logicapp') as g:
        g.custom_command('delete', 'delete_logic_app', confirmation=True)
        g.custom_command('stop', 'stop_webapp')
        g.custom_command('start', 'start_webapp')
        g.custom_command('restart', 'restart_webapp')
        g.generic_update_command('update', getter_name="get_functionapp", setter_name='set_functionapp', exception_handler=update_function_ex_handler_factory(),
                                 custom_func_name='update_functionapp', getter_type=appservice_custom, setter_type=appservice_custom, command_type=webapp_sdk)

    with self.command_group('logicapp', custom_command_type=logicapp_custom) as g:
        g.custom_command('create', 'create_logicapp', exception_handler=ex_handler_factory())
        g.custom_command('list', 'list_logicapp', table_transformer=transform_web_list_output)
        g.custom_show_command('show', 'show_logicapp', table_transformer=transform_web_output)
        g.custom_command('scale', 'scale_logicapp', exception_handler=ex_handler_factory())

    with self.command_group('logicapp config appsettings', custom_command_type=logicapp_custom) as g:
        g.custom_command('list', 'get_logicapp_app_settings', exception_handler=empty_on_404)
        g.custom_command('set', 'update_logicapp_app_settings', exception_handler=ex_handler_factory())
        g.custom_command('delete', 'delete_logicapp_app_settings', exception_handler=ex_handler_factory())

    with self.command_group('logicapp deployment source') as g:
        g.custom_command('config-zip', 'enable_zip_deploy_functionapp')



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
#
# Code generated by aaz-dev-tools
# --------------------------------------------------------------------------------------------

# pylint: skip-file
# flake8: noqa

from azure.cli.core.aaz import *


@register_command(
    "afd custom-domain update",
)
class Update(AAZCommand):
    """Update a new domain within the specified profile.

    :example: Update the custom domain's supported minimum TLS version.
        az afd custom-domain update -g group --custom-domain-name customDomain --profile-name profile --minimum-tls-version TLS12

    :example: Update the custom domain's certificate type to AFD managed certificate.
        az afd custom-domain update -g group --custom-domain-name customDomain --profile-name profile --certificate-type ManagedCertificate
    """

    _aaz_info = {
        "version": "2024-02-01",
        "resources": [
            ["mgmt-plane", "/subscriptions/{}/resourcegroups/{}/providers/microsoft.cdn/profiles/{}/customdomains/{}", "2024-02-01"],
        ]
    }

    AZ_SUPPORT_NO_WAIT = True

    AZ_SUPPORT_GENERIC_UPDATE = True

    def _handler(self, command_args):
        super()._handler(command_args)
        return self.build_lro_poller(self._execute_operations, self._output)

    _args_schema = None

    @classmethod
    def _build_arguments_schema(cls, *args, **kwargs):
        if cls._args_schema is not None:
            return cls._args_schema
        cls._args_schema = super()._build_arguments_schema(*args, **kwargs)

        # define Arg Group ""

        _args_schema = cls._args_schema
        _args_schema.custom_domain_name = AAZStrArg(
            options=["-n", "--name", "--custom-domain-name"],
            help="Name of the domain under the profile which is unique globally.",
            required=True,
            id_part="child_name_1",
        )
        _args_schema.profile_name = AAZStrArg(
            options=["--profile-name"],
            help="Name of the Azure Front Door Standard or Azure Front Door Premium profile which is unique within the resource group.",
            required=True,
            id_part="name",
        )
        _args_schema.resource_group = AAZResourceGroupNameArg(
            required=True,
        )

        # define Arg Group "AzureDnsZone"

        _args_schema = cls._args_schema
        _args_schema.azure_dns_zone = AAZStrArg(
            options=["--azure-dns-zone"],
            arg_group="AzureDnsZone",
            help="ID of the Azure DNS zone.",
            nullable=True,
        )

        # define Arg Group "Properties"

        # define Arg Group "Secret"

        _args_schema = cls._args_schema
        _args_schema.secret = AAZStrArg(
            options=["--secret"],
            arg_group="Secret",
            help="Resource reference to the secret. ie. subs/rg/profile/secret",
            nullable=True,
        )

        # define Arg Group "TlsSettings"

        _args_schema = cls._args_schema
        _args_schema.certificate_type = AAZStrArg(
            options=["--certificate-type"],
            arg_group="TlsSettings",
            help="Defines the source of the SSL certificate.",
            enum={"AzureFirstPartyManagedCertificate": "AzureFirstPartyManagedCertificate", "CustomerCertificate": "CustomerCertificate", "ManagedCertificate": "ManagedCertificate"},
        )
        _args_schema.minimum_tls_version = AAZStrArg(
            options=["--minimum-tls-version"],
            arg_group="TlsSettings",
            help="TLS protocol version that will be used for Https",
            nullable=True,
            enum={"TLS10": "TLS10", "TLS12": "TLS12"},
        )
        return cls._args_schema

    _args_resource_reference_update = None

    @classmethod
    def _build_args_resource_reference_update(cls, _schema):
        if cls._args_resource_reference_update is not None:
            _schema.id = cls._args_resource_reference_update.id
            return

        cls._args_resource_reference_update = AAZObjectArg(
            nullable=True,
        )

        resource_reference_update = cls._args_resource_reference_update
        resource_reference_update.id = AAZStrArg(
            options=["id"],
            help="Resource ID.",
            nullable=True,
        )

        _schema.id = cls._args_resource_reference_update.id

    def _execute_operations(self):
        self.pre_operations()
        self.AFDCustomDomainsGet(ctx=self.ctx)()
        self.pre_instance_update(self.ctx.vars.instance)
        self.InstanceUpdateByJson(ctx=self.ctx)()
        self.InstanceUpdateByGeneric(ctx=self.ctx)()
        self.post_instance_update(self.ctx.vars.instance)
        yield self.AFDCustomDomainsCreate(ctx=self.ctx)()
        self.post_operations()

    @register_callback
    def pre_operations(self):
        pass

    @register_callback
    def post_operations(self):
        pass

    @register_callback
    def pre_instance_update(self, instance):
        pass

    @register_callback
    def post_instance_update(self, instance):
        pass

    def _output(self, *args, **kwargs):
        result = self.deserialize_output(self.ctx.vars.instance, client_flatten=True)
        return result

    class AFDCustomDomainsGet(AAZHttpOperation):
        CLIENT_TYPE = "MgmtClient"

        def __call__(self, *args, **kwargs):
            request = self.make_request()
            session = self.client.send_request(request=request, stream=False, **kwargs)
            if session.http_response.status_code in [200]:
                return self.on_200(session)

            return self.on_error(session.http_response)

        @property
        def url(self):
            return self.client.format_url(
                "/subscriptions/{subscriptionId}/resourceGroups/{resourceGroupName}/providers/Microsoft.Cdn/profiles/{profileName}/customDomains/{customDomainName}",
                **self.url_parameters
            )

        @property
        def method(self):
            return "GET"

        @property
        def error_format(self):
            return "MgmtErrorFormat"

        @property
        def url_parameters(self):
            parameters = {
                **self.serialize_url_param(
                    "customDomainName", self.ctx.args.custom_domain_name,
                    required=True,
                ),
                **self.serialize_url_param(
                    "profileName", self.ctx.args.profile_name,
                    required=True,
                ),
                **self.serialize_url_param(
                    "resourceGroupName", self.ctx.args.resource_group,
                    required=True,
                ),
                **self.serialize_url_param(
                    "subscriptionId", self.ctx.subscription_id,
                    required=True,
                ),
            }
            return parameters

        @property
        def query_parameters(self):
            parameters = {
                **self.serialize_query_param(
                    "api-version", "2024-02-01",
                    required=True,
                ),
            }
            return parameters

        @property
        def header_parameters(self):
            parameters = {
                **self.serialize_header_param(
                    "Accept", "application/json",
                ),
            }
            return parameters

        def on_200(self, session):
            data = self.deserialize_http_content(session)
            self.ctx.set_var(
                "instance",
                data,
                schema_builder=self._build_schema_on_200
            )

        _schema_on_200 = None

        @classmethod
        def _build_schema_on_200(cls):
            if cls._schema_on_200 is not None:
                return cls._schema_on_200

            cls._schema_on_200 = AAZObjectType()
            _UpdateHelper._build_schema_afd_domain_read(cls._schema_on_200)

            return cls._schema_on_200

    class AFDCustomDomainsCreate(AAZHttpOperation):
        CLIENT_TYPE = "MgmtClient"

        def __call__(self, *args, **kwargs):
            request = self.make_request()
            session = self.client.send_request(request=request, stream=False, **kwargs)
            if session.http_response.status_code in [202]:
                return self.client.build_lro_polling(
                    self.ctx.args.no_wait,
                    session,
                    self.on_200_201,
                    self.on_error,
                    lro_options={"final-state-via": "azure-async-operation"},
                    path_format_arguments=self.url_parameters,
                )
            if session.http_response.status_code in [200, 201]:
                return self.client.build_lro_polling(
                    self.ctx.args.no_wait,
                    session,
                    self.on_200_201,
                    self.on_error,
                    lro_options={"final-state-via": "azure-async-operation"},
                    path_format_arguments=self.url_parameters,
                )

            return self.on_error(session.http_response)

        @property
        def url(self):
            return self.client.format_url(
                "/subscriptions/{subscriptionId}/resourceGroups/{resourceGroupName}/providers/Microsoft.Cdn/profiles/{profileName}/customDomains/{customDomainName}",
                **self.url_parameters
            )

        @property
        def method(self):
            return "PUT"

        @property
        def error_format(self):
            return "MgmtErrorFormat"

        @property
        def url_parameters(self):
            parameters = {
                **self.serialize_url_param(
                    "customDomainName", self.ctx.args.custom_domain_name,
                    required=True,
                ),
                **self.serialize_url_param(
                    "profileName", self.ctx.args.profile_name,
                    required=True,
                ),
                **self.serialize_url_param(
                    "resourceGroupName", self.ctx.args.resource_group,
                    required=True,
                ),
                **self.serialize_url_param(
                    "subscriptionId", self.ctx.subscription_id,
                    required=True,
                ),
            }
            return parameters

        @property
        def query_parameters(self):
            parameters = {
                **self.serialize_query_param(
                    "api-version", "2024-02-01",
                    required=True,
                ),
            }
            return parameters

        @property
        def header_parameters(self):
            parameters = {
                **self.serialize_header_param(
                    "Content-Type", "application/json",
                ),
                **self.serialize_header_param(
                    "Accept", "application/json",
                ),
            }
            return parameters

        @property
        def content(self):
            _content_value, _builder = self.new_content_builder(
                self.ctx.args,
                value=self.ctx.vars.instance,
            )

            return self.serialize_content(_content_value)

        def on_200_201(self, session):
            data = self.deserialize_http_content(session)
            self.ctx.set_var(
                "instance",
                data,
                schema_builder=self._build_schema_on_200_201
            )

        _schema_on_200_201 = None

        @classmethod
        def _build_schema_on_200_201(cls):
            if cls._schema_on_200_201 is not None:
                return cls._schema_on_200_201

            cls._schema_on_200_201 = AAZObjectType()
            _UpdateHelper._build_schema_afd_domain_read(cls._schema_on_200_201)

            return cls._schema_on_200_201

    class InstanceUpdateByJson(AAZJsonInstanceUpdateOperation):

        def __call__(self, *args, **kwargs):
            self._update_instance(self.ctx.vars.instance)

        def _update_instance(self, instance):
            _instance_value, _builder = self.new_content_builder(
                self.ctx.args,
                value=instance,
                typ=AAZObjectType
            )
            _builder.set_prop("properties", AAZObjectType, typ_kwargs={"flags": {"client_flatten": True}})

            properties = _builder.get(".properties")
            if properties is not None:
                properties.set_prop("azureDnsZone", AAZObjectType)
                properties.set_prop("tlsSettings", AAZObjectType)

            azure_dns_zone = _builder.get(".properties.azureDnsZone")
            if azure_dns_zone is not None:
                azure_dns_zone.set_prop("id", AAZStrType, ".azure_dns_zone")

            tls_settings = _builder.get(".properties.tlsSettings")
            if tls_settings is not None:
                tls_settings.set_prop("certificateType", AAZStrType, ".certificate_type", typ_kwargs={"flags": {"required": True}})
                tls_settings.set_prop("minimumTlsVersion", AAZStrType, ".minimum_tls_version")
                tls_settings.set_prop("secret", AAZObjectType)

            secret = _builder.get(".properties.tlsSettings.secret")
            if secret is not None:
                secret.set_prop("id", AAZStrType, ".secret")

            return _instance_value

    class InstanceUpdateByGeneric(AAZGenericInstanceUpdateOperation):

        def __call__(self, *args, **kwargs):
            self._update_instance_by_generic(
                self.ctx.vars.instance,
                self.ctx.generic_update_args
            )


class _UpdateHelper:
    """Helper class for Update"""

    @classmethod
    def _build_schema_resource_reference_update(cls, _builder):
        if _builder is None:
            return
        _builder.set_prop("id", AAZStrType, ".id")

    _schema_afd_domain_read = None

    @classmethod
    def _build_schema_afd_domain_read(cls, _schema):
        if cls._schema_afd_domain_read is not None:
            _schema.id = cls._schema_afd_domain_read.id
            _schema.name = cls._schema_afd_domain_read.name
            _schema.properties = cls._schema_afd_domain_read.properties
            _schema.system_data = cls._schema_afd_domain_read.system_data
            _schema.type = cls._schema_afd_domain_read.type
            return

        cls._schema_afd_domain_read = _schema_afd_domain_read = AAZObjectType()

        afd_domain_read = _schema_afd_domain_read
        afd_domain_read.id = AAZStrType(
            flags={"read_only": True},
        )
        afd_domain_read.name = AAZStrType(
            flags={"read_only": True},
        )
        afd_domain_read.properties = AAZObjectType(
            flags={"client_flatten": True},
        )
        afd_domain_read.system_data = AAZObjectType(
            serialized_name="systemData",
            flags={"read_only": True},
        )
        afd_domain_read.type = AAZStrType(
            flags={"read_only": True},
        )

        properties = _schema_afd_domain_read.properties
        properties.azure_dns_zone = AAZObjectType(
            serialized_name="azureDnsZone",
        )
        cls._build_schema_resource_reference_read(properties.azure_dns_zone)
        properties.deployment_status = AAZStrType(
            serialized_name="deploymentStatus",
            flags={"read_only": True},
        )
        properties.domain_validation_state = AAZStrType(
            serialized_name="domainValidationState",
            flags={"read_only": True},
        )
        properties.extended_properties = AAZDictType(
            serialized_name="extendedProperties",
        )
        properties.host_name = AAZStrType(
            serialized_name="hostName",
            flags={"required": True},
        )
        properties.pre_validated_custom_domain_resource_id = AAZObjectType(
            serialized_name="preValidatedCustomDomainResourceId",
        )
        cls._build_schema_resource_reference_read(properties.pre_validated_custom_domain_resource_id)
        properties.profile_name = AAZStrType(
            serialized_name="profileName",
            flags={"read_only": True},
        )
        properties.provisioning_state = AAZStrType(
            serialized_name="provisioningState",
            flags={"read_only": True},
        )
        properties.tls_settings = AAZObjectType(
            serialized_name="tlsSettings",
        )
        properties.validation_properties = AAZObjectType(
            serialized_name="validationProperties",
        )

        extended_properties = _schema_afd_domain_read.properties.extended_properties
        extended_properties.Element = AAZStrType()

        tls_settings = _schema_afd_domain_read.properties.tls_settings
        tls_settings.certificate_type = AAZStrType(
            serialized_name="certificateType",
            flags={"required": True},
        )
        tls_settings.minimum_tls_version = AAZStrType(
            serialized_name="minimumTlsVersion",
        )
        tls_settings.secret = AAZObjectType()
        cls._build_schema_resource_reference_read(tls_settings.secret)

        validation_properties = _schema_afd_domain_read.properties.validation_properties
        validation_properties.expiration_date = AAZStrType(
            serialized_name="expirationDate",
            flags={"read_only": True},
        )
        validation_properties.validation_token = AAZStrType(
            serialized_name="validationToken",
            flags={"read_only": True},
        )

        system_data = _schema_afd_domain_read.system_data
        system_data.created_at = AAZStrType(
            serialized_name="createdAt",
        )
        system_data.created_by = AAZStrType(
            serialized_name="createdBy",
        )
        system_data.created_by_type = AAZStrType(
            serialized_name="createdByType",
        )
        system_data.last_modified_at = AAZStrType(
            serialized_name="lastModifiedAt",
        )
        system_data.last_modified_by = AAZStrType(
            serialized_name="lastModifiedBy",
        )
        system_data.last_modified_by_type = AAZStrType(
            serialized_name="lastModifiedByType",
        )

        _schema.id = cls._schema_afd_domain_read.id
        _schema.name = cls._schema_afd_domain_read.name
        _schema.properties = cls._schema_afd_domain_read.properties
        _schema.system_data = cls._schema_afd_domain_read.system_data
        _schema.type = cls._schema_afd_domain_read.type

    _schema_resource_reference_read = None

    @classmethod
    def _build_schema_resource_reference_read(cls, _schema):
        if cls._schema_resource_reference_read is not None:
            _schema.id = cls._schema_resource_reference_read.id
            return

        cls._schema_resource_reference_read = _schema_resource_reference_read = AAZObjectType()

        resource_reference_read = _schema_resource_reference_read
        resource_reference_read.id = AAZStrType()

        _schema.id = cls._schema_resource_reference_read.id


__all__ = ["Update"]



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
#
# Code generated by aaz-dev-tools
# --------------------------------------------------------------------------------------------

# pylint: skip-file
# flake8: noqa

from azure.cli.core.aaz import *


@register_command(
    "network express-route port wait",
)
class Wait(AAZWaitCommand):
    """Place the CLI in a waiting state until a condition is met.
    """

    _aaz_info = {
        "resources": [
            ["mgmt-plane", "/subscriptions/{}/resourcegroups/{}/providers/microsoft.network/expressrouteports/{}", "2023-09-01"],
        ]
    }

    def _handler(self, command_args):
        super()._handler(command_args)
        self._execute_operations()
        return self._output()

    _args_schema = None

    @classmethod
    def _build_arguments_schema(cls, *args, **kwargs):
        if cls._args_schema is not None:
            return cls._args_schema
        cls._args_schema = super()._build_arguments_schema(*args, **kwargs)

        # define Arg Group ""

        _args_schema = cls._args_schema
        _args_schema.name = AAZStrArg(
            options=["-n", "--name"],
            help="ExpressRoute port name.",
            required=True,
            id_part="name",
        )
        _args_schema.resource_group = AAZResourceGroupNameArg(
            required=True,
        )
        return cls._args_schema

    def _execute_operations(self):
        self.pre_operations()
        self.ExpressRoutePortsGet(ctx=self.ctx)()
        self.post_operations()

    @register_callback
    def pre_operations(self):
        pass

    @register_callback
    def post_operations(self):
        pass

    def _output(self, *args, **kwargs):
        result = self.deserialize_output(self.ctx.vars.instance, client_flatten=False)
        return result

    class ExpressRoutePortsGet(AAZHttpOperation):
        CLIENT_TYPE = "MgmtClient"

        def __call__(self, *args, **kwargs):
            request = self.make_request()
            session = self.client.send_request(request=request, stream=False, **kwargs)
            if session.http_response.status_code in [200]:
                return self.on_200(session)

            return self.on_error(session.http_response)

        @property
        def url(self):
            return self.client.format_url(
                "/subscriptions/{subscriptionId}/resourceGroups/{resourceGroupName}/providers/Microsoft.Network/ExpressRoutePorts/{expressRoutePortName}",
                **self.url_parameters
            )

        @property
        def method(self):
            return "GET"

        @property
        def error_format(self):
            return "ODataV4Format"

        @property
        def url_parameters(self):
            parameters = {
                **self.serialize_url_param(
                    "expressRoutePortName", self.ctx.args.name,
                    required=True,
                ),
                **self.serialize_url_param(
                    "resourceGroupName", self.ctx.args.resource_group,
                    required=True,
                ),
                **self.serialize_url_param(
                    "subscriptionId", self.ctx.subscription_id,
                    required=True,
                ),
            }
            return parameters

        @property
        def query_parameters(self):
            parameters = {
                **self.serialize_query_param(
                    "api-version", "2023-09-01",
                    required=True,
                ),
            }
            return parameters

        @property
        def header_parameters(self):
            parameters = {
                **self.serialize_header_param(
                    "Accept", "application/json",
                ),
            }
            return parameters

        def on_200(self, session):
            data = self.deserialize_http_content(session)
            self.ctx.set_var(
                "instance",
                data,
                schema_builder=self._build_schema_on_200
            )

        _schema_on_200 = None

        @classmethod
        def _build_schema_on_200(cls):
            if cls._schema_on_200 is not None:
                return cls._schema_on_200

            cls._schema_on_200 = AAZObjectType()

            _schema_on_200 = cls._schema_on_200
            _schema_on_200.etag = AAZStrType(
                flags={"read_only": True},
            )
            _schema_on_200.id = AAZStrType()
            _schema_on_200.identity = AAZObjectType()
            _schema_on_200.location = AAZStrType()
            _schema_on_200.name = AAZStrType(
                flags={"read_only": True},
            )
            _schema_on_200.properties = AAZObjectType(
                flags={"client_flatten": True},
            )
            _schema_on_200.tags = AAZDictType()
            _schema_on_200.type = AAZStrType(
                flags={"read_only": True},
            )

            identity = cls._schema_on_200.identity
            identity.principal_id = AAZStrType(
                serialized_name="principalId",
                flags={"read_only": True},
            )
            identity.tenant_id = AAZStrType(
                serialized_name="tenantId",
                flags={"read_only": True},
            )
            identity.type = AAZStrType()
            identity.user_assigned_identities = AAZDictType(
                serialized_name="userAssignedIdentities",
            )

            user_assigned_identities = cls._schema_on_200.identity.user_assigned_identities
            user_assigned_identities.Element = AAZObjectType()

            _element = cls._schema_on_200.identity.user_assigned_identities.Element
            _element.client_id = AAZStrType(
                serialized_name="clientId",
                flags={"read_only": True},
            )
            _element.principal_id = AAZStrType(
                serialized_name="principalId",
                flags={"read_only": True},
            )

            properties = cls._schema_on_200.properties
            properties.allocation_date = AAZStrType(
                serialized_name="allocationDate",
                flags={"read_only": True},
            )
            properties.bandwidth_in_gbps = AAZIntType(
                serialized_name="bandwidthInGbps",
            )
            properties.billing_type = AAZStrType(
                serialized_name="billingType",
            )
            properties.circuits = AAZListType(
                flags={"read_only": True},
            )
            properties.encapsulation = AAZStrType()
            properties.ether_type = AAZStrType(
                serialized_name="etherType",
                flags={"read_only": True},
            )
            properties.links = AAZListType()
            properties.mtu = AAZStrType(
                flags={"read_only": True},
            )
            properties.peering_location = AAZStrType(
                serialized_name="peeringLocation",
            )
            properties.provisioned_bandwidth_in_gbps = AAZFloatType(
                serialized_name="provisionedBandwidthInGbps",
                flags={"read_only": True},
            )
            properties.provisioning_state = AAZStrType(
                serialized_name="provisioningState",
                flags={"read_only": True},
            )
            properties.resource_guid = AAZStrType(
                serialized_name="resourceGuid",
                flags={"read_only": True},
            )

            circuits = cls._schema_on_200.properties.circuits
            circuits.Element = AAZObjectType()

            _element = cls._schema_on_200.properties.circuits.Element
            _element.id = AAZStrType()

            links = cls._schema_on_200.properties.links
            links.Element = AAZObjectType()

            _element = cls._schema_on_200.properties.links.Element
            _element.etag = AAZStrType(
                flags={"read_only": True},
            )
            _element.id = AAZStrType()
            _element.name = AAZStrType()
            _element.properties = AAZObjectType(
                flags={"client_flatten": True},
            )

            properties = cls._schema_on_200.properties.links.Element.properties
            properties.admin_state = AAZStrType(
                serialized_name="adminState",
            )
            properties.colo_location = AAZStrType(
                serialized_name="coloLocation",
                flags={"read_only": True},
            )
            properties.connector_type = AAZStrType(
                serialized_name="connectorType",
                flags={"read_only": True},
            )
            properties.interface_name = AAZStrType(
                serialized_name="interfaceName",
                flags={"read_only": True},
            )
            properties.mac_sec_config = AAZObjectType(
                serialized_name="macSecConfig",
            )
            properties.patch_panel_id = AAZStrType(
                serialized_name="patchPanelId",
                flags={"read_only": True},
            )
            properties.provisioning_state = AAZStrType(
                serialized_name="provisioningState",
                flags={"read_only": True},
            )
            properties.rack_id = AAZStrType(
                serialized_name="rackId",
                flags={"read_only": True},
            )
            properties.router_name = AAZStrType(
                serialized_name="routerName",
                flags={"read_only": True},
            )

            mac_sec_config = cls._schema_on_200.properties.links.Element.properties.mac_sec_config
            mac_sec_config.cak_secret_identifier = AAZStrType(
                serialized_name="cakSecretIdentifier",
            )
            mac_sec_config.cipher = AAZStrType()
            mac_sec_config.ckn_secret_identifier = AAZStrType(
                serialized_name="cknSecretIdentifier",
            )
            mac_sec_config.sci_state = AAZStrType(
                serialized_name="sciState",
            )

            tags = cls._schema_on_200.tags
            tags.Element = AAZStrType()

            return cls._schema_on_200


class _WaitHelper:
    """Helper class for Wait"""


__all__ = ["Wait"]



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------

from knack.help_files import helps

helps['bot'] = """
    type: group
    short-summary: Manage Microsoft Azure Bot Service.
"""
helps['bot create'] = """
    type: command
    short-summary: Create a new v4 SDK bot.
    long-summary: Create a new v4 SDK bot.
"""
helps['bot show'] = """
    type: command
    short-summary: Get an existing bot.
    long-summary: Get information about an existing bot. To get the information needed to connect to the bot, use the
                  --msbot flag with the command.
    examples:
    - name: Get the information needed to connect to an existing bot on Azure
      text: |-
        az bot show -n botName -g MyResourceGroup --msbot
"""
helps['bot prepare-publish'] = """
    type: command
    short-summary: (Maintenance mode) Add scripts to your local source code directory to
                   be able to publish back using `az bot publish` for v3 SDK bots.
"""
helps['bot prepare-deploy'] = """
    type: command
    short-summary: Add scripts/config files for publishing with `az webapp deployment`.
    long-summary: Add scripts or configuration files to the root of your local source code directory to be able to
                  publish using `az webapp deployment`. When your code is deployed to your App Service, the generated
                  scripts or configuration files should be appear in D:\\home\\site\\wwwroot on App Service's Kudu web
                  page.
    examples:
        - name: Prepare to use `az webapp` to deploy a Javascript bot by fetching a Node.js IIS web.config file.
          text: |-
            az bot prepare-deploy --lang Javascript --code-dir "MyBotCode"
        - name: Prepare to use `az webapp` to deploy a Csharp bot by creating a .deployment file.
          text: |-
            az bot prepare-deploy --lang Csharp --code-dir "." --proj-file-path "MyBot.csproj"
"""
helps['bot delete'] = """
    type: command
    short-summary: Delete an existing bot.
"""
helps['bot update'] = """
    type: command
    short-summary: Update an existing bot.
    examples:
        - name: Update description on a bot
          text: |-
            az bot update -n botName -g MyResourceGroup --endpoint "https://bing.com/api/messages" --display-name "Hello World"
"""
helps['bot publish'] = """
    type: command
    short-summary: Publish to a bot's associated app service.
    long-summary: Publish your source code to your bot's associated app service. This is DEPRECATED for v4 bots and no
                  longer recommended for publishing v4 bots to Azure. Instead use `az bot prepare-deploy` and `az webapp
                  deployment` to deploy your v4 bot. For more information see https://aka.ms/deploy-your-bot.
    examples:
        - name: Publish source code to your Azure App, from within the bot code folder
          text: |-
            az bot publish -n botName -g MyResourceGroup
"""
helps['bot download'] = """
    type: command
    short-summary: Download an existing bot.
    long-summary: The source code is downloaded from the web app associated with the bot.
                  You can then make changes to it and publish it back to your app.
"""
helps['bot facebook create'] = """
    type: command
    short-summary: Create the Facebook Channel on a bot.
    examples:
        - name: Create the Facebook Channel for a bot
          text: |
            az bot facebook create -n botName -g MyResourceGroup --appid myAppId \\
            --page-id myPageId --secret mySecret --token myToken
"""
helps['bot email create'] = """
    type: command
    short-summary: Create the Email Channel on a bot.
    examples:
        - name: Create the Email Channel for a bot
          text: |-
            az bot email create -n botName -g MyResourceGroup -a abc@outlook.com \\
            -p password
"""
helps['bot msteams create'] = """
    type: command
    short-summary: Create the Microsoft Teams Channel on a bot.
    examples:
        - name: Create the Microsoft Teams Channel for a bot with calling enabled
          text: |-
            az bot msteams create -n botName -g MyResourceGroup --enable-calling
            --calling-web-hook https://www.myapp.com/
"""
helps['bot skype create'] = """
    type: command
    short-summary: Create the Skype Channel on a bot.
    examples:
        - name: Create the Skype Channel for a bot with messaging and screen sharing enabled
          text: |-
            az bot skype create -n botName -g MyResourceGroup --enable-messaging
            --enable-screen-sharing
"""
helps['bot kik create'] = """
    type: command
    short-summary: Create the Kik Channel on a bot.
    examples:
        - name: Create the Kik Channel for a bot.
          text: |-
            az bot kik create -n botName -g MyResourceGroup -u mykikname \\
            --key key --is-validated
"""
helps['bot directline create'] = """
    type: command
    short-summary: Create the DirectLine Channel on a bot with only v3 protocol enabled.
    examples:
        - name: Create the DirectLine Channel for a bot.
          text: |-
            az bot directline create -n botName -g MyResourceGroup --disablev1
"""
helps['bot directline update'] = """
    type: command
    short-summary: Update the DirectLine Channel on a bot with only v3 protocol enabled.
    examples:
        - name: Update the DirectLine Channel for a bot.
          text: |-
            az bot directline update -n botName -g MyResourceGroup --disablev1
"""
helps['bot telegram create'] = """
    type: command
    short-summary: Create the Telegram Channel on a bot.
    examples:
        - name: Create the Telegram Channel for a bot.
          text: |-
            az bot telegram create -n botName -g MyResourceGroup --access-token token
            --is-validated
"""
helps['bot sms create'] = """
    type: command
    short-summary: Create the SMS Channel on a bot.
    examples:
        - name: Create the SMS Channel for a bot.
          text: |-
            az bot sms create -n botName -g MyResourceGroup --account-sid sid \\
            --auth-token token --is-validated --phone 1234567890
"""
helps['bot slack create'] = """
    type: command
    short-summary: Create the Slack Channel on a bot.
    examples:
        - name: Create the Slack Channel for a bot.
          text: |-
            az bot slack create -n botName -g MyResourceGroup --client-id clientid \\
            --client-secret secret --verification-token token
"""
helps['bot authsetting'] = """
    type: group
    short-summary: Manage OAuth connection settings on a bot.
"""
helps['bot authsetting create'] = """
    type: command
    short-summary: Create an OAuth connection setting on a bot.
    examples:
        - name: Create a new OAuth connection setting on a bot.
          text: |-
            az bot authsetting create -g MyResourceGroup -n botName -c myConnectionName \\
            --client-id clientId --client-secret secret --provider-scope-string "scope1 scope2"\\
            --service google --parameters id=myid
"""
helps['bot authsetting show'] = """
    type: command
    short-summary: Show details of an OAuth connection setting on a bot.
"""
helps['bot authsetting list'] = """
    type: command
    short-summary: Show all OAuth connection settings on a bot.
"""
helps['bot authsetting delete'] = """
    type: command
    short-summary: Delete an OAuth connection setting on a bot.
"""
helps['bot authsetting list-providers'] = """
    type: command
    short-summary: List details for all service providers available for creating OAuth connection settings.
    examples:
        - name: List all service providers.
          text: |-
            az bot authsetting list-providers
        - name: Filter by a particular type of service provider.
          text: |-
            az bot authsetting list-providers --provider-name google
"""


for channel in ['Facebook', 'email', 'Skype', 'Kik', 'Directline', 'Telegram', 'SMS', 'Slack']:
    channel_name = channel.lower()
    helps['bot {0} delete'.format(channel_name)] = """
        type: command
        short-summary: Delete the {0} Channel on a bot
    """.format(channel)
    helps['bot {0} show'.format(channel_name)] = """
        type: command
        short-summary: Get details of the {0} Channel on a bot
    """.format(channel)
    helps['bot {0}'.format(channel_name)] = """
        type: group
        short-summary: Manage the {0} Channel on a bot.
    """.format(channel)


helps['bot msteams delete'] = """
    type: command
    short-summary: Delete the Microsoft Teams Channel on a bot
"""
helps['bot msteams show'] = """
    type: command
    short-summary: Get details of the Microsoft Teams Channel on a bot
"""
helps['bot msteams'] = """
    type: group
    short-summary: Manage the Microsoft Teams Channel on a bot.
"""
helps['bot webchat show'] = """
    type: command
    short-summary: Get details of the Webchat Channel on a bot
"""
helps['bot webchat'] = """
    type: group
    short-summary: Manage the Webchat Channel on a bot.
"""



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
#
# Code generated by aaz-dev-tools
# --------------------------------------------------------------------------------------------

# pylint: skip-file
# flake8: noqa

from azure.cli.core.aaz import *


@register_command(
    "afd rule show",
)
class Show(AAZCommand):
    """Get an existing delivery rule within a rule set.

    :example: show details of the delivery rule with name rule1.
        az afd rule show -g group --rule-set-name ruleSetName --profile-name profile --rule-name rule1
    """

    _aaz_info = {
        "version": "2024-02-01",
        "resources": [
            ["mgmt-plane", "/subscriptions/{}/resourcegroups/{}/providers/microsoft.cdn/profiles/{}/rulesets/{}/rules/{}", "2024-02-01"],
        ]
    }

    def _handler(self, command_args):
        super()._handler(command_args)
        self._execute_operations()
        return self._output()

    _args_schema = None

    @classmethod
    def _build_arguments_schema(cls, *args, **kwargs):
        if cls._args_schema is not None:
            return cls._args_schema
        cls._args_schema = super()._build_arguments_schema(*args, **kwargs)

        # define Arg Group ""

        _args_schema = cls._args_schema
        _args_schema.profile_name = AAZStrArg(
            options=["--profile-name"],
            help="Name of the Azure Front Door Standard or Azure Front Door Premium profile which is unique within the resource group.",
            required=True,
            id_part="name",
        )
        _args_schema.resource_group = AAZResourceGroupNameArg(
            required=True,
        )
        _args_schema.rule_name = AAZStrArg(
            options=["-n", "--name", "--rule-name"],
            help="Name of the delivery rule which is unique within the endpoint.",
            required=True,
            id_part="child_name_2",
        )
        _args_schema.rule_set_name = AAZStrArg(
            options=["--rule-set-name"],
            help="Name of the rule set under the profile.",
            required=True,
            id_part="child_name_1",
        )
        return cls._args_schema

    def _execute_operations(self):
        self.pre_operations()
        self.RulesGet(ctx=self.ctx)()
        self.post_operations()

    @register_callback
    def pre_operations(self):
        pass

    @register_callback
    def post_operations(self):
        pass

    def _output(self, *args, **kwargs):
        result = self.deserialize_output(self.ctx.vars.instance, client_flatten=True)
        return result

    class RulesGet(AAZHttpOperation):
        CLIENT_TYPE = "MgmtClient"

        def __call__(self, *args, **kwargs):
            request = self.make_request()
            session = self.client.send_request(request=request, stream=False, **kwargs)
            if session.http_response.status_code in [200]:
                return self.on_200(session)

            return self.on_error(session.http_response)

        @property
        def url(self):
            return self.client.format_url(
                "/subscriptions/{subscriptionId}/resourceGroups/{resourceGroupName}/providers/Microsoft.Cdn/profiles/{profileName}/ruleSets/{ruleSetName}/rules/{ruleName}",
                **self.url_parameters
            )

        @property
        def method(self):
            return "GET"

        @property
        def error_format(self):
            return "MgmtErrorFormat"

        @property
        def url_parameters(self):
            parameters = {
                **self.serialize_url_param(
                    "profileName", self.ctx.args.profile_name,
                    required=True,
                ),
                **self.serialize_url_param(
                    "resourceGroupName", self.ctx.args.resource_group,
                    required=True,
                ),
                **self.serialize_url_param(
                    "ruleName", self.ctx.args.rule_name,
                    required=True,
                ),
                **self.serialize_url_param(
                    "ruleSetName", self.ctx.args.rule_set_name,
                    required=True,
                ),
                **self.serialize_url_param(
                    "subscriptionId", self.ctx.subscription_id,
                    required=True,
                ),
            }
            return parameters

        @property
        def query_parameters(self):
            parameters = {
                **self.serialize_query_param(
                    "api-version", "2024-02-01",
                    required=True,
                ),
            }
            return parameters

        @property
        def header_parameters(self):
            parameters = {
                **self.serialize_header_param(
                    "Accept", "application/json",
                ),
            }
            return parameters

        def on_200(self, session):
            data = self.deserialize_http_content(session)
            self.ctx.set_var(
                "instance",
                data,
                schema_builder=self._build_schema_on_200
            )

        _schema_on_200 = None

        @classmethod
        def _build_schema_on_200(cls):
            if cls._schema_on_200 is not None:
                return cls._schema_on_200

            cls._schema_on_200 = AAZObjectType()

            _schema_on_200 = cls._schema_on_200
            _schema_on_200.id = AAZStrType(
                flags={"read_only": True},
            )
            _schema_on_200.name = AAZStrType(
                flags={"read_only": True},
            )
            _schema_on_200.properties = AAZObjectType(
                flags={"client_flatten": True},
            )
            _schema_on_200.system_data = AAZObjectType(
                serialized_name="systemData",
                flags={"read_only": True},
            )
            _schema_on_200.type = AAZStrType(
                flags={"read_only": True},
            )

            properties = cls._schema_on_200.properties
            properties.actions = AAZListType(
                flags={"required": True},
            )
            properties.conditions = AAZListType()
            properties.deployment_status = AAZStrType(
                serialized_name="deploymentStatus",
                flags={"read_only": True},
            )
            properties.match_processing_behavior = AAZStrType(
                serialized_name="matchProcessingBehavior",
            )
            properties.order = AAZIntType(
                flags={"required": True},
            )
            properties.provisioning_state = AAZStrType(
                serialized_name="provisioningState",
                flags={"read_only": True},
            )
            properties.rule_set_name = AAZStrType(
                serialized_name="ruleSetName",
                flags={"read_only": True},
            )

            actions = cls._schema_on_200.properties.actions
            actions.Element = AAZObjectType()

            _element = cls._schema_on_200.properties.actions.Element
            _element.name = AAZStrType(
                flags={"required": True},
            )

            disc_cache_expiration = cls._schema_on_200.properties.actions.Element.discriminate_by("name", "CacheExpiration")
            disc_cache_expiration.parameters = AAZObjectType(
                flags={"required": True},
            )

            parameters = cls._schema_on_200.properties.actions.Element.discriminate_by("name", "CacheExpiration").parameters
            parameters.cache_behavior = AAZStrType(
                serialized_name="cacheBehavior",
                flags={"required": True},
            )
            parameters.cache_duration = AAZStrType(
                serialized_name="cacheDuration",
                nullable=True,
            )
            parameters.cache_type = AAZStrType(
                serialized_name="cacheType",
                flags={"required": True},
            )
            parameters.type_name = AAZStrType(
                serialized_name="typeName",
                flags={"required": True},
            )

            disc_cache_key_query_string = cls._schema_on_200.properties.actions.Element.discriminate_by("name", "CacheKeyQueryString")
            disc_cache_key_query_string.parameters = AAZObjectType(
                flags={"required": True},
            )

            parameters = cls._schema_on_200.properties.actions.Element.discriminate_by("name", "CacheKeyQueryString").parameters
            parameters.query_parameters = AAZStrType(
                serialized_name="queryParameters",
                nullable=True,
            )
            parameters.query_string_behavior = AAZStrType(
                serialized_name="queryStringBehavior",
                flags={"required": True},
            )
            parameters.type_name = AAZStrType(
                serialized_name="typeName",
                flags={"required": True},
            )

            disc_modify_request_header = cls._schema_on_200.properties.actions.Element.discriminate_by("name", "ModifyRequestHeader")
            disc_modify_request_header.parameters = AAZObjectType(
                flags={"required": True},
            )
            _ShowHelper._build_schema_header_action_parameters_read(disc_modify_request_header.parameters)

            disc_modify_response_header = cls._schema_on_200.properties.actions.Element.discriminate_by("name", "ModifyResponseHeader")
            disc_modify_response_header.parameters = AAZObjectType(
                flags={"required": True},
            )
            _ShowHelper._build_schema_header_action_parameters_read(disc_modify_response_header.parameters)

            disc_origin_group_override = cls._schema_on_200.properties.actions.Element.discriminate_by("name", "OriginGroupOverride")
            disc_origin_group_override.parameters = AAZObjectType(
                flags={"required": True},
            )

            parameters = cls._schema_on_200.properties.actions.Element.discriminate_by("name", "OriginGroupOverride").parameters
            parameters.origin_group = AAZObjectType(
                serialized_name="originGroup",
                flags={"required": True},
            )
            _ShowHelper._build_schema_resource_reference_read(parameters.origin_group)
            parameters.type_name = AAZStrType(
                serialized_name="typeName",
                flags={"required": True},
            )

            disc_route_configuration_override = cls._schema_on_200.properties.actions.Element.discriminate_by("name", "RouteConfigurationOverride")
            disc_route_configuration_override.parameters = AAZObjectType(
                flags={"required": True},
            )

            parameters = cls._schema_on_200.properties.actions.Element.discriminate_by("name", "RouteConfigurationOverride").parameters
            parameters.cache_configuration = AAZObjectType(
                serialized_name="cacheConfiguration",
            )
            parameters.origin_group_override = AAZObjectType(
                serialized_name="originGroupOverride",
            )
            parameters.type_name = AAZStrType(
                serialized_name="typeName",
                flags={"required": True},
            )

            cache_configuration = cls._schema_on_200.properties.actions.Element.discriminate_by("name", "RouteConfigurationOverride").parameters.cache_configuration
            cache_configuration.cache_behavior = AAZStrType(
                serialized_name="cacheBehavior",
            )
            cache_configuration.cache_duration = AAZStrType(
                serialized_name="cacheDuration",
            )
            cache_configuration.is_compression_enabled = AAZStrType(
                serialized_name="isCompressionEnabled",
            )
            cache_configuration.query_parameters = AAZStrType(
                serialized_name="queryParameters",
            )
            cache_configuration.query_string_caching_behavior = AAZStrType(
                serialized_name="queryStringCachingBehavior",
            )

            origin_group_override = cls._schema_on_200.properties.actions.Element.discriminate_by("name", "RouteConfigurationOverride").parameters.origin_group_override
            origin_group_override.forwarding_protocol = AAZStrType(
                serialized_name="forwardingProtocol",
            )
            origin_group_override.origin_group = AAZObjectType(
                serialized_name="originGroup",
            )
            _ShowHelper._build_schema_resource_reference_read(origin_group_override.origin_group)

            disc_url_redirect = cls._schema_on_200.properties.actions.Element.discriminate_by("name", "UrlRedirect")
            disc_url_redirect.parameters = AAZObjectType(
                flags={"required": True},
            )

            parameters = cls._schema_on_200.properties.actions.Element.discriminate_by("name", "UrlRedirect").parameters
            parameters.custom_fragment = AAZStrType(
                serialized_name="customFragment",
            )
            parameters.custom_hostname = AAZStrType(
                serialized_name="customHostname",
            )
            parameters.custom_path = AAZStrType(
                serialized_name="customPath",
            )
            parameters.custom_query_string = AAZStrType(
                serialized_name="customQueryString",
            )
            parameters.destination_protocol = AAZStrType(
                serialized_name="destinationProtocol",
            )
            parameters.redirect_type = AAZStrType(
                serialized_name="redirectType",
                flags={"required": True},
            )
            parameters.type_name = AAZStrType(
                serialized_name="typeName",
                flags={"required": True},
            )

            disc_url_rewrite = cls._schema_on_200.properties.actions.Element.discriminate_by("name", "UrlRewrite")
            disc_url_rewrite.parameters = AAZObjectType(
                flags={"required": True},
            )

            parameters = cls._schema_on_200.properties.actions.Element.discriminate_by("name", "UrlRewrite").parameters
            parameters.destination = AAZStrType(
                flags={"required": True},
            )
            parameters.preserve_unmatched_path = AAZBoolType(
                serialized_name="preserveUnmatchedPath",
            )
            parameters.source_pattern = AAZStrType(
                serialized_name="sourcePattern",
                flags={"required": True},
            )
            parameters.type_name = AAZStrType(
                serialized_name="typeName",
                flags={"required": True},
            )

            disc_url_signing = cls._schema_on_200.properties.actions.Element.discriminate_by("name", "UrlSigning")
            disc_url_signing.parameters = AAZObjectType(
                flags={"required": True},
            )

            parameters = cls._schema_on_200.properties.actions.Element.discriminate_by("name", "UrlSigning").parameters
            parameters.algorithm = AAZStrType()
            parameters.parameter_name_override = AAZListType(
                serialized_name="parameterNameOverride",
            )
            parameters.type_name = AAZStrType(
                serialized_name="typeName",
                flags={"required": True},
            )

            parameter_name_override = cls._schema_on_200.properties.actions.Element.discriminate_by("name", "UrlSigning").parameters.parameter_name_override
            parameter_name_override.Element = AAZObjectType()

            _element = cls._schema_on_200.properties.actions.Element.discriminate_by("name", "UrlSigning").parameters.parameter_name_override.Element
            _element.param_indicator = AAZStrType(
                serialized_name="paramIndicator",
                flags={"required": True},
            )
            _element.param_name = AAZStrType(
                serialized_name="paramName",
                flags={"required": True},
            )

            conditions = cls._schema_on_200.properties.conditions
            conditions.Element = AAZObjectType()

            _element = cls._schema_on_200.properties.conditions.Element
            _element.name = AAZStrType(
                flags={"required": True},
            )

            disc_client_port = cls._schema_on_200.properties.conditions.Element.discriminate_by("name", "ClientPort")
            disc_client_port.parameters = AAZObjectType(
                flags={"required": True},
            )

            parameters = cls._schema_on_200.properties.conditions.Element.discriminate_by("name", "ClientPort").parameters
            parameters.match_values = AAZListType(
                serialized_name="matchValues",
            )
            parameters.negate_condition = AAZBoolType(
                serialized_name="negateCondition",
            )
            parameters.operator = AAZStrType(
                flags={"required": True},
            )
            parameters.transforms = AAZListType()
            parameters.type_name = AAZStrType(
                serialized_name="typeName",
                flags={"required": True},
            )

            match_values = cls._schema_on_200.properties.conditions.Element.discriminate_by("name", "ClientPort").parameters.match_values
            match_values.Element = AAZStrType()

            transforms = cls._schema_on_200.properties.conditions.Element.discriminate_by("name", "ClientPort").parameters.transforms
            transforms.Element = AAZStrType()

            disc_cookies = cls._schema_on_200.properties.conditions.Element.discriminate_by("name", "Cookies")
            disc_cookies.parameters = AAZObjectType(
                flags={"required": True},
            )

            parameters = cls._schema_on_200.properties.conditions.Element.discriminate_by("name", "Cookies").parameters
            parameters.match_values = AAZListType(
                serialized_name="matchValues",
            )
            parameters.negate_condition = AAZBoolType(
                serialized_name="negateCondition",
            )
            parameters.operator = AAZStrType(
                flags={"required": True},
            )
            parameters.selector = AAZStrType()
            parameters.transforms = AAZListType()
            parameters.type_name = AAZStrType(
                serialized_name="typeName",
                flags={"required": True},
            )

            match_values = cls._schema_on_200.properties.conditions.Element.discriminate_by("name", "Cookies").parameters.match_values
            match_values.Element = AAZStrType()

            transforms = cls._schema_on_200.properties.conditions.Element.discriminate_by("name", "Cookies").parameters.transforms
            transforms.Element = AAZStrType()

            disc_host_name = cls._schema_on_200.properties.conditions.Element.discriminate_by("name", "HostName")
            disc_host_name.parameters = AAZObjectType(
                flags={"required": True},
            )

            parameters = cls._schema_on_200.properties.conditions.Element.discriminate_by("name", "HostName").parameters
            parameters.match_values = AAZListType(
                serialized_name="matchValues",
            )
            parameters.negate_condition = AAZBoolType(
                serialized_name="negateCondition",
            )
            parameters.operator = AAZStrType(
                flags={"required": True},
            )
            parameters.transforms = AAZListType()
            parameters.type_name = AAZStrType(
                serialized_name="typeName",
                flags={"required": True},
            )

            match_values = cls._schema_on_200.properties.conditions.Element.discriminate_by("name", "HostName").parameters.match_values
            match_values.Element = AAZStrType()

            transforms = cls._schema_on_200.properties.conditions.Element.discriminate_by("name", "HostName").parameters.transforms
            transforms.Element = AAZStrType()

            disc_http_version = cls._schema_on_200.properties.conditions.Element.discriminate_by("name", "HttpVersion")
            disc_http_version.parameters = AAZObjectType(
                flags={"required": True},
            )

            parameters = cls._schema_on_200.properties.conditions.Element.discriminate_by("name", "HttpVersion").parameters
            parameters.match_values = AAZListType(
                serialized_name="matchValues",
            )
            parameters.negate_condition = AAZBoolType(
                serialized_name="negateCondition",
            )
            parameters.operator = AAZStrType(
                flags={"required": True},
            )
            parameters.transforms = AAZListType()
            parameters.type_name = AAZStrType(
                serialized_name="typeName",
                flags={"required": True},
            )

            match_values = cls._schema_on_200.properties.conditions.Element.discriminate_by("name", "HttpVersion").parameters.match_values
            match_values.Element = AAZStrType()

            transforms = cls._schema_on_200.properties.conditions.Element.discriminate_by("name", "HttpVersion").parameters.transforms
            transforms.Element = AAZStrType()

            disc_is_device = cls._schema_on_200.properties.conditions.Element.discriminate_by("name", "IsDevice")
            disc_is_device.parameters = AAZObjectType(
                flags={"required": True},
            )

            parameters = cls._schema_on_200.properties.conditions.Element.discriminate_by("name", "IsDevice").parameters
            parameters.match_values = AAZListType(
                serialized_name="matchValues",
            )
            parameters.negate_condition = AAZBoolType(
                serialized_name="negateCondition",
            )
            parameters.operator = AAZStrType(
                flags={"required": True},
            )
            parameters.transforms = AAZListType()
            parameters.type_name = AAZStrType(
                serialized_name="typeName",
                flags={"required": True},
            )

            match_values = cls._schema_on_200.properties.conditions.Element.discriminate_by("name", "IsDevice").parameters.match_values
            match_values.Element = AAZStrType()

            transforms = cls._schema_on_200.properties.conditions.Element.discriminate_by("name", "IsDevice").parameters.transforms
            transforms.Element = AAZStrType()

            disc_post_args = cls._schema_on_200.properties.conditions.Element.discriminate_by("name", "PostArgs")
            disc_post_args.parameters = AAZObjectType(
                flags={"required": True},
            )

            parameters = cls._schema_on_200.properties.conditions.Element.discriminate_by("name", "PostArgs").parameters
            parameters.match_values = AAZListType(
                serialized_name="matchValues",
            )
            parameters.negate_condition = AAZBoolType(
                serialized_name="negateCondition",
            )
            parameters.operator = AAZStrType(
                flags={"required": True},
            )
            parameters.selector = AAZStrType()
            parameters.transforms = AAZListType()
            parameters.type_name = AAZStrType(
                serialized_name="typeName",
                flags={"required": True},
            )

            match_values = cls._schema_on_200.properties.conditions.Element.discriminate_by("name", "PostArgs").parameters.match_values
            match_values.Element = AAZStrType()

            transforms = cls._schema_on_200.properties.conditions.Element.discriminate_by("name", "PostArgs").parameters.transforms
            transforms.Element = AAZStrType()

            disc_query_string = cls._schema_on_200.properties.conditions.Element.discriminate_by("name", "QueryString")
            disc_query_string.parameters = AAZObjectType(
                flags={"required": True},
            )

            parameters = cls._schema_on_200.properties.conditions.Element.discriminate_by("name", "QueryString").parameters
            parameters.match_values = AAZListType(
                serialized_name="matchValues",
            )
            parameters.negate_condition = AAZBoolType(
                serialized_name="negateCondition",
            )
            parameters.operator = AAZStrType(
                flags={"required": True},
            )
            parameters.transforms = AAZListType()
            parameters.type_name = AAZStrType(
                serialized_name="typeName",
                flags={"required": True},
            )

            match_values = cls._schema_on_200.properties.conditions.Element.discriminate_by("name", "QueryString").parameters.match_values
            match_values.Element = AAZStrType()

            transforms = cls._schema_on_200.properties.conditions.Element.discriminate_by("name", "QueryString").parameters.transforms
            transforms.Element = AAZStrType()

            disc_remote_address = cls._schema_on_200.properties.conditions.Element.discriminate_by("name", "RemoteAddress")
            disc_remote_address.parameters = AAZObjectType(
                flags={"required": True},
            )

            parameters = cls._schema_on_200.properties.conditions.Element.discriminate_by("name", "RemoteAddress").parameters
            parameters.match_values = AAZListType(
                serialized_name="matchValues",
            )
            parameters.negate_condition = AAZBoolType(
                serialized_name="negateCondition",
            )
            parameters.operator = AAZStrType(
                flags={"required": True},
            )
            parameters.transforms = AAZListType()
            parameters.type_name = AAZStrType(
                serialized_name="typeName",
                flags={"required": True},
            )

            match_values = cls._schema_on_200.properties.conditions.Element.discriminate_by("name", "RemoteAddress").parameters.match_values
            match_values.Element = AAZStrType()

            transforms = cls._schema_on_200.properties.conditions.Element.discriminate_by("name", "RemoteAddress").parameters.transforms
            transforms.Element = AAZStrType()

            disc_request_body = cls._schema_on_200.properties.conditions.Element.discriminate_by("name", "RequestBody")
            disc_request_body.parameters = AAZObjectType(
                flags={"required": True},
            )

            parameters = cls._schema_on_200.properties.conditions.Element.discriminate_by("name", "RequestBody").parameters
            parameters.match_values = AAZListType(
                serialized_name="matchValues",
            )
            parameters.negate_condition = AAZBoolType(
                serialized_name="negateCondition",
            )
            parameters.operator = AAZStrType(
                flags={"required": True},
            )
            parameters.transforms = AAZListType()
            parameters.type_name = AAZStrType(
                serialized_name="typeName",
                flags={"required": True},
            )

            match_values = cls._schema_on_200.properties.conditions.Element.discriminate_by("name", "RequestBody").parameters.match_values
            match_values.Element = AAZStrType()

            transforms = cls._schema_on_200.properties.conditions.Element.discriminate_by("name", "RequestBody").parameters.transforms
            transforms.Element = AAZStrType()

            disc_request_header = cls._schema_on_200.properties.conditions.Element.discriminate_by("name", "RequestHeader")
            disc_request_header.parameters = AAZObjectType(
                flags={"required": True},
            )

            parameters = cls._schema_on_200.properties.conditions.Element.discriminate_by("name", "RequestHeader").parameters
            parameters.match_values = AAZListType(
                serialized_name="matchValues",
            )
            parameters.negate_condition = AAZBoolType(
                serialized_name="negateCondition",
            )
            parameters.operator = AAZStrType(
                flags={"required": True},
            )
            parameters.selector = AAZStrType()
            parameters.transforms = AAZListType()
            parameters.type_name = AAZStrType(
                serialized_name="typeName",
                flags={"required": True},
            )

            match_values = cls._schema_on_200.properties.conditions.Element.discriminate_by("name", "RequestHeader").parameters.match_values
            match_values.Element = AAZStrType()

            transforms = cls._schema_on_200.properties.conditions.Element.discriminate_by("name", "RequestHeader").parameters.transforms
            transforms.Element = AAZStrType()

            disc_request_method = cls._schema_on_200.properties.conditions.Element.discriminate_by("name", "RequestMethod")
            disc_request_method.parameters = AAZObjectType(
                flags={"required": True},
            )

            parameters = cls._schema_on_200.properties.conditions.Element.discriminate_by("name", "RequestMethod").parameters
            parameters.match_values = AAZListType(
                serialized_name="matchValues",
            )
            parameters.negate_condition = AAZBoolType(
                serialized_name="negateCondition",
            )
            parameters.operator = AAZStrType(
                flags={"required": True},
            )
            parameters.transforms = AAZListType()
            parameters.type_name = AAZStrType(
                serialized_name="typeName",
                flags={"required": True},
            )

            match_values = cls._schema_on_200.properties.conditions.Element.discriminate_by("name", "RequestMethod").parameters.match_values
            match_values.Element = AAZStrType()

            transforms = cls._schema_on_200.properties.conditions.Element.discriminate_by("name", "RequestMethod").parameters.transforms
            transforms.Element = AAZStrType()

            disc_request_scheme = cls._schema_on_200.properties.conditions.Element.discriminate_by("name", "RequestScheme")
            disc_request_scheme.parameters = AAZObjectType(
                flags={"required": True},
            )

            parameters = cls._schema_on_200.properties.conditions.Element.discriminate_by("name", "RequestScheme").parameters
            parameters.match_values = AAZListType(
                serialized_name="matchValues",
            )
            parameters.negate_condition = AAZBoolType(
                serialized_name="negateCondition",
            )
            parameters.operator = AAZStrType(
                flags={"required": True},
            )
            parameters.transforms = AAZListType()
            parameters.type_name = AAZStrType(
                serialized_name="typeName",
                flags={"required": True},
            )

            match_values = cls._schema_on_200.properties.conditions.Element.discriminate_by("name", "RequestScheme").parameters.match_values
            match_values.Element = AAZStrType()

            transforms = cls._schema_on_200.properties.conditions.Element.discriminate_by("name", "RequestScheme").parameters.transforms
            transforms.Element = AAZStrType()

            disc_request_uri = cls._schema_on_200.properties.conditions.Element.discriminate_by("name", "RequestUri")
            disc_request_uri.parameters = AAZObjectType(
                flags={"required": True},
            )

            parameters = cls._schema_on_200.properties.conditions.Element.discriminate_by("name", "RequestUri").parameters
            parameters.match_values = AAZListType(
                serialized_name="matchValues",
            )
            parameters.negate_condition = AAZBoolType(
                serialized_name="negateCondition",
            )
            parameters.operator = AAZStrType(
                flags={"required": True},
            )
            parameters.transforms = AAZListType()
            parameters.type_name = AAZStrType(
                serialized_name="typeName",
                flags={"required": True},
            )

            match_values = cls._schema_on_200.properties.conditions.Element.discriminate_by("name", "RequestUri").parameters.match_values
            match_values.Element = AAZStrType()

            transforms = cls._schema_on_200.properties.conditions.Element.discriminate_by("name", "RequestUri").parameters.transforms
            transforms.Element = AAZStrType()

            disc_server_port = cls._schema_on_200.properties.conditions.Element.discriminate_by("name", "ServerPort")
            disc_server_port.parameters = AAZObjectType(
                flags={"required": True},
            )

            parameters = cls._schema_on_200.properties.conditions.Element.discriminate_by("name", "ServerPort").parameters
            parameters.match_values = AAZListType(
                serialized_name="matchValues",
            )
            parameters.negate_condition = AAZBoolType(
                serialized_name="negateCondition",
            )
            parameters.operator = AAZStrType(
                flags={"required": True},
            )
            parameters.transforms = AAZListType()
            parameters.type_name = AAZStrType(
                serialized_name="typeName",
                flags={"required": True},
            )

            match_values = cls._schema_on_200.properties.conditions.Element.discriminate_by("name", "ServerPort").parameters.match_values
            match_values.Element = AAZStrType()

            transforms = cls._schema_on_200.properties.conditions.Element.discriminate_by("name", "ServerPort").parameters.transforms
            transforms.Element = AAZStrType()

            disc_socket_addr = cls._schema_on_200.properties.conditions.Element.discriminate_by("name", "SocketAddr")
            disc_socket_addr.parameters = AAZObjectType(
                flags={"required": True},
            )

            parameters = cls._schema_on_200.properties.conditions.Element.discriminate_by("name", "SocketAddr").parameters
            parameters.match_values = AAZListType(
                serialized_name="matchValues",
            )
            parameters.negate_condition = AAZBoolType(
                serialized_name="negateCondition",
            )
            parameters.operator = AAZStrType(
                flags={"required": True},
            )
            parameters.transforms = AAZListType()
            parameters.type_name = AAZStrType(
                serialized_name="typeName",
                flags={"required": True},
            )

            match_values = cls._schema_on_200.properties.conditions.Element.discriminate_by("name", "SocketAddr").parameters.match_values
            match_values.Element = AAZStrType()

            transforms = cls._schema_on_200.properties.conditions.Element.discriminate_by("name", "SocketAddr").parameters.transforms
            transforms.Element = AAZStrType()

            disc_ssl_protocol = cls._schema_on_200.properties.conditions.Element.discriminate_by("name", "SslProtocol")
            disc_ssl_protocol.parameters = AAZObjectType(
                flags={"required": True},
            )

            parameters = cls._schema_on_200.properties.conditions.Element.discriminate_by("name", "SslProtocol").parameters
            parameters.match_values = AAZListType(
                serialized_name="matchValues",
            )
            parameters.negate_condition = AAZBoolType(
                serialized_name="negateCondition",
            )
            parameters.operator = AAZStrType(
                flags={"required": True},
            )
            parameters.transforms = AAZListType()
            parameters.type_name = AAZStrType(
                serialized_name="typeName",
                flags={"required": True},
            )

            match_values = cls._schema_on_200.properties.conditions.Element.discriminate_by("name", "SslProtocol").parameters.match_values
            match_values.Element = AAZStrType()

            transforms = cls._schema_on_200.properties.conditions.Element.discriminate_by("name", "SslProtocol").parameters.transforms
            transforms.Element = AAZStrType()

            disc_url_file_extension = cls._schema_on_200.properties.conditions.Element.discriminate_by("name", "UrlFileExtension")
            disc_url_file_extension.parameters = AAZObjectType(
                flags={"required": True},
            )

            parameters = cls._schema_on_200.properties.conditions.Element.discriminate_by("name", "UrlFileExtension").parameters
            parameters.match_values = AAZListType(
                serialized_name="matchValues",
            )
            parameters.negate_condition = AAZBoolType(
                serialized_name="negateCondition",
            )
            parameters.operator = AAZStrType(
                flags={"required": True},
            )
            parameters.transforms = AAZListType()
            parameters.type_name = AAZStrType(
                serialized_name="typeName",
                flags={"required": True},
            )

            match_values = cls._schema_on_200.properties.conditions.Element.discriminate_by("name", "UrlFileExtension").parameters.match_values
            match_values.Element = AAZStrType()

            transforms = cls._schema_on_200.properties.conditions.Element.discriminate_by("name", "UrlFileExtension").parameters.transforms
            transforms.Element = AAZStrType()

            disc_url_file_name = cls._schema_on_200.properties.conditions.Element.discriminate_by("name", "UrlFileName")
            disc_url_file_name.parameters = AAZObjectType(
                flags={"required": True},
            )

            parameters = cls._schema_on_200.properties.conditions.Element.discriminate_by("name", "UrlFileName").parameters
            parameters.match_values = AAZListType(
                serialized_name="matchValues",
            )
            parameters.negate_condition = AAZBoolType(
                serialized_name="negateCondition",
            )
            parameters.operator = AAZStrType(
                flags={"required": True},
            )
            parameters.transforms = AAZListType()
            parameters.type_name = AAZStrType(
                serialized_name="typeName",
                flags={"required": True},
            )

            match_values = cls._schema_on_200.properties.conditions.Element.discriminate_by("name", "UrlFileName").parameters.match_values
            match_values.Element = AAZStrType()

            transforms = cls._schema_on_200.properties.conditions.Element.discriminate_by("name", "UrlFileName").parameters.transforms
            transforms.Element = AAZStrType()

            disc_url_path = cls._schema_on_200.properties.conditions.Element.discriminate_by("name", "UrlPath")
            disc_url_path.parameters = AAZObjectType(
                flags={"required": True},
            )

            parameters = cls._schema_on_200.properties.conditions.Element.discriminate_by("name", "UrlPath").parameters
            parameters.match_values = AAZListType(
                serialized_name="matchValues",
            )
            parameters.negate_condition = AAZBoolType(
                serialized_name="negateCondition",
            )
            parameters.operator = AAZStrType(
                flags={"required": True},
            )
            parameters.transforms = AAZListType()
            parameters.type_name = AAZStrType(
                serialized_name="typeName",
                flags={"required": True},
            )

            match_values = cls._schema_on_200.properties.conditions.Element.discriminate_by("name", "UrlPath").parameters.match_values
            match_values.Element = AAZStrType()

            transforms = cls._schema_on_200.properties.conditions.Element.discriminate_by("name", "UrlPath").parameters.transforms
            transforms.Element = AAZStrType()

            system_data = cls._schema_on_200.system_data
            system_data.created_at = AAZStrType(
                serialized_name="createdAt",
            )
            system_data.created_by = AAZStrType(
                serialized_name="createdBy",
            )
            system_data.created_by_type = AAZStrType(
                serialized_name="createdByType",
            )
            system_data.last_modified_at = AAZStrType(
                serialized_name="lastModifiedAt",
            )
            system_data.last_modified_by = AAZStrType(
                serialized_name="lastModifiedBy",
            )
            system_data.last_modified_by_type = AAZStrType(
                serialized_name="lastModifiedByType",
            )

            return cls._schema_on_200


class _ShowHelper:
    """Helper class for Show"""

    _schema_header_action_parameters_read = None

    @classmethod
    def _build_schema_header_action_parameters_read(cls, _schema):
        if cls._schema_header_action_parameters_read is not None:
            _schema.header_action = cls._schema_header_action_parameters_read.header_action
            _schema.header_name = cls._schema_header_action_parameters_read.header_name
            _schema.type_name = cls._schema_header_action_parameters_read.type_name
            _schema.value = cls._schema_header_action_parameters_read.value
            return

        cls._schema_header_action_parameters_read = _schema_header_action_parameters_read = AAZObjectType()

        header_action_parameters_read = _schema_header_action_parameters_read
        header_action_parameters_read.header_action = AAZStrType(
            serialized_name="headerAction",
            flags={"required": True},
        )
        header_action_parameters_read.header_name = AAZStrType(
            serialized_name="headerName",
            flags={"required": True},
        )
        header_action_parameters_read.type_name = AAZStrType(
            serialized_name="typeName",
            flags={"required": True},
        )
        header_action_parameters_read.value = AAZStrType()

        _schema.header_action = cls._schema_header_action_parameters_read.header_action
        _schema.header_name = cls._schema_header_action_parameters_read.header_name
        _schema.type_name = cls._schema_header_action_parameters_read.type_name
        _schema.value = cls._schema_header_action_parameters_read.value

    _schema_resource_reference_read = None

    @classmethod
    def _build_schema_resource_reference_read(cls, _schema):
        if cls._schema_resource_reference_read is not None:
            _schema.id = cls._schema_resource_reference_read.id
            return

        cls._schema_resource_reference_read = _schema_resource_reference_read = AAZObjectType()

        resource_reference_read = _schema_resource_reference_read
        resource_reference_read.id = AAZStrType()

        _schema.id = cls._schema_resource_reference_read.id


__all__ = ["Show"]



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
#
# Code generated by aaz-dev-tools
# --------------------------------------------------------------------------------------------

# pylint: skip-file
# flake8: noqa

from azure.cli.core.aaz import *


@register_command(
    "billing profile reservation list",
)
class List(AAZCommand):
    """List the reservations for a billing profile and the roll up counts of reservations group by provisioning state.
    """

    _aaz_info = {
        "version": "2020-05-01",
        "resources": [
            ["mgmt-plane", "/providers/microsoft.billing/billingaccounts/{}/billingprofiles/{}/reservations", "2020-05-01"],
        ]
    }

    AZ_SUPPORT_PAGINATION = True

    def _handler(self, command_args):
        super()._handler(command_args)
        return self.build_paging(self._execute_operations, self._output)

    _args_schema = None

    @classmethod
    def _build_arguments_schema(cls, *args, **kwargs):
        if cls._args_schema is not None:
            return cls._args_schema
        cls._args_schema = super()._build_arguments_schema(*args, **kwargs)

        # define Arg Group ""

        _args_schema = cls._args_schema
        _args_schema.billing_account_name = AAZStrArg(
            options=["--billing-account-name"],
            help="The ID that uniquely identifies a billing account.",
            required=True,
        )
        _args_schema.billing_profile_name = AAZStrArg(
            options=["--billing-profile-name"],
            help="The ID that uniquely identifies a billing profile.",
            required=True,
        )
        _args_schema.filter = AAZStrArg(
            options=["--filter"],
            help="May be used to filter by reservation properties. The filter supports 'eq', 'or', and 'and'. It does not currently support 'ne', 'gt', 'le', 'ge', or 'not'.",
        )
        _args_schema.orderby = AAZStrArg(
            options=["--orderby"],
            help="May be used to sort order by reservation properties.",
        )
        _args_schema.refresh_summary = AAZStrArg(
            options=["--refresh-summary"],
            help="To indicate whether to refresh the roll up counts of the reservations group by provisioning state",
        )
        _args_schema.selected_state = AAZStrArg(
            options=["--selected-state"],
            help="The selected provisioning state",
        )
        return cls._args_schema

    def _execute_operations(self):
        self.pre_operations()
        self.ReservationsListByBillingProfile(ctx=self.ctx)()
        self.post_operations()

    @register_callback
    def pre_operations(self):
        pass

    @register_callback
    def post_operations(self):
        pass

    def _output(self, *args, **kwargs):
        result = self.deserialize_output(self.ctx.vars.instance.value, client_flatten=True)
        next_link = self.deserialize_output(self.ctx.vars.instance.next_link)
        return result, next_link

    class ReservationsListByBillingProfile(AAZHttpOperation):
        CLIENT_TYPE = "MgmtClient"

        def __call__(self, *args, **kwargs):
            request = self.make_request()
            session = self.client.send_request(request=request, stream=False, **kwargs)
            if session.http_response.status_code in [200]:
                return self.on_200(session)

            return self.on_error(session.http_response)

        @property
        def url(self):
            return self.client.format_url(
                "/providers/Microsoft.Billing/billingAccounts/{billingAccountName}/billingProfiles/{billingProfileName}/reservations",
                **self.url_parameters
            )

        @property
        def method(self):
            return "GET"

        @property
        def error_format(self):
            return "ODataV4Format"

        @property
        def url_parameters(self):
            parameters = {
                **self.serialize_url_param(
                    "billingAccountName", self.ctx.args.billing_account_name,
                    required=True,
                ),
                **self.serialize_url_param(
                    "billingProfileName", self.ctx.args.billing_profile_name,
                    required=True,
                ),
            }
            return parameters

        @property
        def query_parameters(self):
            parameters = {
                **self.serialize_query_param(
                    "$filter", self.ctx.args.filter,
                ),
                **self.serialize_query_param(
                    "$orderby", self.ctx.args.orderby,
                ),
                **self.serialize_query_param(
                    "refreshSummary", self.ctx.args.refresh_summary,
                ),
                **self.serialize_query_param(
                    "selectedState", self.ctx.args.selected_state,
                ),
                **self.serialize_query_param(
                    "api-version", "2020-05-01",
                    required=True,
                ),
            }
            return parameters

        @property
        def header_parameters(self):
            parameters = {
                **self.serialize_header_param(
                    "Accept", "application/json",
                ),
            }
            return parameters

        def on_200(self, session):
            data = self.deserialize_http_content(session)
            self.ctx.set_var(
                "instance",
                data,
                schema_builder=self._build_schema_on_200
            )

        _schema_on_200 = None

        @classmethod
        def _build_schema_on_200(cls):
            if cls._schema_on_200 is not None:
                return cls._schema_on_200

            cls._schema_on_200 = AAZObjectType()

            _schema_on_200 = cls._schema_on_200
            _schema_on_200.next_link = AAZStrType(
                serialized_name="nextLink",
                flags={"read_only": True},
            )
            _schema_on_200.summary = AAZObjectType()
            _schema_on_200.value = AAZListType(
                flags={"read_only": True},
            )

            summary = cls._schema_on_200.summary
            summary.cancelled_count = AAZFloatType(
                serialized_name="cancelledCount",
                flags={"read_only": True},
            )
            summary.expired_count = AAZFloatType(
                serialized_name="expiredCount",
                flags={"read_only": True},
            )
            summary.expiring_count = AAZFloatType(
                serialized_name="expiringCount",
                flags={"read_only": True},
            )
            summary.failed_count = AAZFloatType(
                serialized_name="failedCount",
                flags={"read_only": True},
            )
            summary.pending_count = AAZFloatType(
                serialized_name="pendingCount",
                flags={"read_only": True},
            )
            summary.succeeded_count = AAZFloatType(
                serialized_name="succeededCount",
                flags={"read_only": True},
            )

            value = cls._schema_on_200.value
            value.Element = AAZObjectType()

            _element = cls._schema_on_200.value.Element
            _element.id = AAZStrType(
                flags={"read_only": True},
            )
            _element.location = AAZStrType(
                flags={"read_only": True},
            )
            _element.name = AAZStrType(
                flags={"read_only": True},
            )
            _element.properties = AAZObjectType(
                flags={"client_flatten": True},
            )
            _element.sku = AAZObjectType()
            _element.type = AAZStrType(
                flags={"read_only": True},
            )

            properties = cls._schema_on_200.value.Element.properties
            properties.applied_scope_type = AAZStrType(
                serialized_name="appliedScopeType",
                flags={"read_only": True},
            )
            properties.applied_scopes = AAZListType(
                serialized_name="appliedScopes",
            )
            properties.display_name = AAZStrType(
                serialized_name="displayName",
                flags={"read_only": True},
            )
            properties.display_provisioning_state = AAZStrType(
                serialized_name="displayProvisioningState",
                flags={"read_only": True},
            )
            properties.effective_date_time = AAZStrType(
                serialized_name="effectiveDateTime",
                flags={"read_only": True},
            )
            properties.expiry_date = AAZStrType(
                serialized_name="expiryDate",
                flags={"read_only": True},
            )
            properties.provisioning_state = AAZStrType(
                serialized_name="provisioningState",
                flags={"read_only": True},
            )
            properties.provisioning_sub_state = AAZStrType(
                serialized_name="provisioningSubState",
                flags={"read_only": True},
            )
            properties.quantity = AAZFloatType(
                flags={"read_only": True},
            )
            properties.renew = AAZBoolType(
                flags={"read_only": True},
            )
            properties.renew_source = AAZStrType(
                serialized_name="renewSource",
                flags={"read_only": True},
            )
            properties.reserved_resource_type = AAZStrType(
                serialized_name="reservedResourceType",
                flags={"read_only": True},
            )
            properties.sku_description = AAZStrType(
                serialized_name="skuDescription",
                flags={"read_only": True},
            )
            properties.term = AAZStrType(
                flags={"read_only": True},
            )
            properties.user_friendly_applied_scope_type = AAZStrType(
                serialized_name="userFriendlyAppliedScopeType",
                flags={"read_only": True},
            )
            properties.user_friendly_renew_state = AAZStrType(
                serialized_name="userFriendlyRenewState",
                flags={"read_only": True},
            )
            properties.utilization = AAZObjectType(
                flags={"read_only": True},
            )

            applied_scopes = cls._schema_on_200.value.Element.properties.applied_scopes
            applied_scopes.Element = AAZStrType(
                flags={"read_only": True},
            )

            utilization = cls._schema_on_200.value.Element.properties.utilization
            utilization.aggregates = AAZListType()
            utilization.trend = AAZStrType(
                flags={"read_only": True},
            )

            aggregates = cls._schema_on_200.value.Element.properties.utilization.aggregates
            aggregates.Element = AAZObjectType()

            _element = cls._schema_on_200.value.Element.properties.utilization.aggregates.Element
            _element.grain = AAZFloatType(
                flags={"read_only": True},
            )
            _element.grain_unit = AAZStrType(
                serialized_name="grainUnit",
                flags={"read_only": True},
            )
            _element.value = AAZFloatType(
                flags={"read_only": True},
            )
            _element.value_unit = AAZStrType(
                serialized_name="valueUnit",
                flags={"read_only": True},
            )

            sku = cls._schema_on_200.value.Element.sku
            sku.name = AAZStrType(
                flags={"read_only": True},
            )

            return cls._schema_on_200


class _ListHelper:
    """Helper class for List"""


__all__ = ["List"]
